from __future__ import absolute_import

from time import sleep
from django.core.urlresolvers import reverse
from django.test import TestCase

from nodeshot.core.base.tests import user_fixtures
from nodeshot.core.nodes.models import Node
from nodeshot.ui.default import settings as local_settings

from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.common.by import By


class DefaultUiSeleniumTest(TestCase):
    fixtures = [
        'initial_data.json',
        user_fixtures,
        'test_layers.json',
        'test_status.json',
        'test_nodes.json',
        'test_images.json',
        'test_pages.json'
    ]

    INDEX_URL = '%s%s' % (local_settings.settings.SITE_URL, reverse('ui:index'))
    LEAFLET_MAP = 'Ns.body.currentView.content.currentView.map'

    def _wait_until_ajax_complete(self, seconds, message):
        """ waits until all ajax requests are complete """
        def condition(driver):
            return driver.execute_script("return typeof(jQuery) !== 'undefined' && jQuery.active === 0")
        WebDriverWait(self.browser, seconds).until(condition, message)

    def _wait_until_element_visible(self, selector, seconds, message):
        """ waits until css selector is present and visible """
        WebDriverWait(self.browser, seconds).until(
            expected_conditions.visibility_of_element_located((By.CSS_SELECTOR, selector)),
            message
        )

    def _hashchange(self, hash):
        self.browser.get('%s%s' % (self.INDEX_URL, hash))
        self._wait_until_ajax_complete(10, 'Timeout')

    def _reset(self):
        """ reset and reload browser (clear localstorage and go to index) """
        self._hashchange('#/')
        self.browser.execute_script('localStorage.clear()')
        self.browser.delete_all_cookies()
        self.browser.refresh()
        self._wait_until_ajax_complete(10, 'Timeout')

    def _login(self, username='admin', password='tester', open_modal=True):
        if open_modal:
            # open sign in modal
            self.browser.find_element_by_css_selector('#main-actions a[data-target="#signin-modal"]').click()
            self._wait_until_element_visible('#js-signin-form', 3, 'signin modal not visible')
        # insert credentials
        username_field = self.browser.find_element_by_css_selector('#js-signin-form input[name=username]')
        username_field.clear()
        username_field.send_keys(username)
        password_field = self.browser.find_element_by_css_selector('#js-signin-form input[name=password]')
        password_field.clear()
        password_field.send_keys(password)
        # log in
        self.browser.find_element_by_css_selector('#js-signin-form button.btn-default').click()
        self._wait_until_ajax_complete(5, 'timeout')
        self._wait_until_element_visible('#js-username', 3, 'username after login not visible')
        # check username
        self.assertEqual(self.browser.find_element_by_css_selector('#js-username').text, 'admin')

    def _logout(self):
        # open account menu
        self.browser.find_element_by_css_selector('#js-username').click()
        # log out
        self._wait_until_element_visible('#js-logout', 3, '#js-logout not visible')
        self.browser.find_element_by_css_selector('#js-logout').click()
        self._wait_until_ajax_complete(5, 'timeout')
        # ensure UI has gone back to initial state
        self._wait_until_element_visible('#main-actions a[data-target="#signin-modal"]', 3, 'sign in link not visible after logout')

    @classmethod
    def setUpClass(cls):
        cls.browser = webdriver.Firefox()
        cls.browser.get(cls.INDEX_URL)
        super(DefaultUiSeleniumTest, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        cls.browser.quit()
        super(DefaultUiSeleniumTest, cls).tearDownClass()

    def setUp(self):
        """ reset browser to initial state """
        browser = self.browser
        browser.execute_script('localStorage.clear()')
        browser.delete_all_cookies()
        browser.execute_script("Ns.db.user.clear(); Ns.db.user.trigger('logout');")
        browser.set_window_size(1100, 700)

    def test_home(self):
        self._hashchange('#')
        self.assertEqual(self.browser.find_element_by_css_selector('article.center-stage h1').text, 'Home')
        self.assertTrue(self.browser.execute_script('return Ns.db.pages.get("home") !== undefined'))
        self.assertEqual(self.browser.title, 'Home - Nodeshot')

    def test_menu_already_active(self):
        self._hashchange('#')
        # ensure clicking multiple times on a level1 menu item  still displays it as active
        self.browser.find_element_by_css_selector('#nav-bar li.active a').click()
        self.browser.find_element_by_css_selector('#nav-bar li.active a').click()
        # ensure the same on a nested menu item
        self.browser.find_element_by_css_selector('#nav-bar a.dropdown-toggle').click()
        self.browser.find_element_by_css_selector("a[href='#pages/about']").click()
        self._wait_until_ajax_complete(1, 'page timeout')
        self.browser.find_element_by_css_selector('#nav-bar li.active a.dropdown-toggle').click()
        self.browser.find_element_by_css_selector("a[href='#pages/about']").click()
        self._wait_until_ajax_complete(1, 'page timeout')
        self.browser.find_element_by_css_selector('#nav-bar li.active a.dropdown-toggle').click()

    def test_map(self):
        self._reset()
        self._hashchange('#map')
        browser = self.browser
        LEAFLET_MAP = self.LEAFLET_MAP

        # test title
        self.assertEqual(browser.title, 'Map - Nodeshot')

        # basic test
        self.assertTrue(browser.execute_script("return Ns.body.currentView.$el.attr('id') == 'map-container'"))
        browser.find_element_by_css_selector('#map-js.leaflet-container')
        self.assertTrue(browser.execute_script("return %s._leaflet_id > -1" % LEAFLET_MAP))

        # layers control
        browser.find_element_by_css_selector('#map-js .leaflet-control-layers-list')
        LAYERS_CONTROLS = 'return _.values(%s.layerscontrol._layers)' % LEAFLET_MAP
        self.assertEqual(browser.execute_script('%s[0].name' % LAYERS_CONTROLS), 'Map')
        self.assertEqual(browser.execute_script('%s[1].name' % LAYERS_CONTROLS), 'Satellite')

        # ensure coordinates are correct
        self.assertEqual(browser.execute_script("return %s.getZoom()" % LEAFLET_MAP), local_settings.MAP_ZOOM)
        # leaflet coordinates are approximated when zoom is low, so let's check Nodeshot JS settings
        self.assertEqual(browser.execute_script("return Ns.settings.map.lat"), local_settings.MAP_CENTER[0])
        self.assertEqual(browser.execute_script("return Ns.settings.map.lng"), local_settings.MAP_CENTER[1])
        self.assertEqual(browser.execute_script("return Ns.settings.map.zoom"), local_settings.MAP_ZOOM)

        # ensure rememberCoordinates() works
        browser.execute_script("%s.setView([42.111111, 12.111111], 17)" % LEAFLET_MAP)
        sleep(0.5)
        self._hashchange('#')
        self.assertEqual(browser.execute_script("return localStorage.getObject('map').lat.toString().substr(0, 7)"), "42.1111")
        self.assertEqual(browser.execute_script("return localStorage.getObject('map').lng.toString().substr(0, 7)"), "12.1111")
        self.assertEqual(browser.execute_script("return localStorage.getObject('map').zoom"), 17)
        self._hashchange('#/map')
        self.assertEqual(browser.execute_script("return %s.getZoom()" % LEAFLET_MAP), 17)
        self.assertEqual(browser.execute_script("return %s.getCenter().lat.toString().substr(0, 7)" % LEAFLET_MAP), "42.1111")
        self.assertEqual(browser.execute_script("return %s.getCenter().lng.toString().substr(0, 7)" % LEAFLET_MAP), "12.1111")

        # map is resized when window is resized
        window_size = browser.get_window_size()

        map_size = {}
        map_size['width'] = browser.execute_script("return $('#map-js').width()")
        map_size['height'] = browser.execute_script("return $('#map-js').height()")
        browser.set_window_size(window_size['width'] - 10, window_size['height'] - 10)
        sleep(0.2)
        self.assertEqual(browser.execute_script("return $('#map-js').width()"), map_size['width'] - 10)
        self.assertEqual(browser.execute_script("return $('#map-js').height()"), map_size['height'] - 10)
        browser.set_window_size(window_size['width'], window_size['height'])
        sleep(0.2)
        self.assertEqual(browser.execute_script("return $('#map-js').width()"), map_size['width'])
        self.assertEqual(browser.execute_script("return $('#map-js').height()"), map_size['height'])

        # esnure data has been loaded
        map_objects = browser.execute_script('return Ns.body.currentView.content.currentView.collection.length')
        nodes = Node.objects.published().access_level_up_to('public').count()
        self.assertEqual(map_objects, nodes)

        # ensure clustering works
        browser.execute_script("%s.setView([42.001111, 12.611111], 7)" % LEAFLET_MAP)
        self.assertEqual(browser.execute_script('return $("#map-js .cluster.marker-active").length'), 1)
        self.assertEqual(browser.execute_script('return $("#map-js .cluster.marker-potential").length'), 1)

    def test_map_data_caching_and_reload(self):
        self._reset()
        browser = self.browser
        # ensure Ns.db.geo is empty
        self.assertTrue(browser.execute_script('return Ns.db.geo.isEmpty() === true'))
        self._hashchange('#map')
        # ensure Ns.db.geo is not empty anymore (cached)
        self.assertTrue(browser.execute_script('return Ns.db.geo.isEmpty() === false'))
        # ensure node with higher ACL not visible
        self.assertTrue(browser.execute_script("return Ns.db.geo.get('hidden-rome') === undefined"))
        # log in as admin
        self._login()
        # ensure node with higher ACL is now visible
        self.assertFalse(browser.execute_script("return Ns.db.geo.get('hidden-rome') === undefined"))
        # ensure data is not duplicated
        browser.execute_script('%s.setView([41.8879, 12.50621], 13)' % self.LEAFLET_MAP)
        browser.execute_script('$("#map-control-layer-rome").trigger("click")')
        self.assertEqual(len(browser.find_elements_by_css_selector('#map-js g')), 0)
        # log out
        self._logout()
        # ensure node with higher ACL not visible anymore
        self.assertTrue(browser.execute_script("return Ns.db.geo.get('hidden-rome') === undefined"))

    def test_map_popup(self):
        self._reset()
        self._hashchange('#/map/pomezia')
        browser = self.browser
        # changing url fragment opens popup
        self._wait_until_element_visible('#map-js .leaflet-popup-content-wrapper', 1, 'popup not visible')
        self.assertEqual(len(browser.find_elements_by_css_selector('#map-js .leaflet-popup-content-wrapper')), 1)
        self.assertEqual(browser.find_element_by_css_selector('#map-js .leaflet-popup-content-wrapper h4').text, 'Pomezia')

        # refreshing goes back there
        browser.refresh()
        self._wait_until_element_visible('#map-js .leaflet-popup-content-wrapper', 1, 'popup not visible')
        self.assertEqual(len(browser.find_elements_by_css_selector('#map-js .leaflet-popup-content-wrapper')), 1)
        self.assertEqual(browser.find_element_by_css_selector('#map-js .leaflet-popup-content-wrapper h4').text, 'Pomezia')

        # changing fragment goes to another element
        self._hashchange('#/map/fusolab')
        self._wait_until_element_visible('#map-js .leaflet-popup-content-wrapper', 1, 'popup not visible')
        self.assertEqual(len(browser.find_elements_by_css_selector('#map-js .leaflet-popup-content-wrapper')), 1)
        self.assertEqual(browser.find_element_by_css_selector('#map-js .leaflet-popup-content-wrapper h4').text, 'Fusolab Rome')

        # back button supported
        browser.back()
        self._wait_until_element_visible('#map-js .leaflet-popup-content-wrapper', 1, 'popup not visible')
        self.assertEqual(browser.find_element_by_css_selector('#map-js .leaflet-popup-content-wrapper h4').text, 'Pomezia')

        # forward button supported
        browser.forward()
        self._wait_until_element_visible('#map-js .leaflet-popup-content-wrapper', 1, 'popup not visible')
        self.assertEqual(browser.find_element_by_css_selector('#map-js .leaflet-popup-content-wrapper h4').text, 'Fusolab Rome')

        # close button changes url fragment to "map"
        browser.find_element_by_css_selector('#map-js .leaflet-popup-close-button').click()
        sleep(0.3)
        self.assertEqual(browser.current_url.split('#')[1], 'map')
        # go back
        browser.back()
        self.assertIn('map/fusolab', browser.current_url.split('#')[1])
        sleep(0.2)
        self.assertEqual(browser.find_element_by_css_selector('#map-js .leaflet-popup-content-wrapper h4').text, 'Fusolab Rome')
        # go forward and expect popup to be closed
        browser.forward()
        self.assertEqual(browser.current_url.split('#')[1], 'map')
        self.assertEqual(len(browser.find_elements_by_css_selector('#map-js .leaflet-popup-content-wrapper')), 0)

        # got to map url, no popups open
        self._hashchange('#map')
        self.assertEqual(browser.current_url.split('#')[1], 'map')
        self.assertEqual(len(browser.find_elements_by_css_selector('#map-js .leaflet-popup-content-wrapper')), 0)
        # simulate clicking on a popup
        browser.find_elements_by_css_selector('#map-js g')[0].click()
        # ensure popup is open
        self._wait_until_element_visible('#map-js .leaflet-popup-content-wrapper', 1, 'popup not visible')
        # url must be longer than 4 chars, eg: "map/something"
        self.assertTrue(len(browser.current_url.split('#')[1]) > 4)
        # go back
        self._hashchange('#map')

        # does not exist case
        self._hashchange('#/map/wrong')
        sleep(0.25)
        self.assertEqual(browser.find_element_by_css_selector('#tmp-modal').is_displayed(), True)
        browser.find_element_by_css_selector('#tmp-modal button').click()

    def test_map_toolbar(self):
        self._hashchange('#/map')
        browser = self.browser

        # ensure rendered
        self.assertGreater(len(browser.find_elements_by_css_selector('#map-toolbar a')), 4)
        self.assertEqual(len(browser.find_elements_by_css_selector('#map-toolbar')), 1)

        # test base layers
        view = self.LEAFLET_MAP.replace('.map', '')
        self.assertEqual(browser.execute_script("return Boolean(%s.baseLayers.Map._map)" % view), True)
        self.assertEqual(browser.execute_script("return Boolean(%s.baseLayers.Satellite._map)" % view), False)
        self.assertEqual(browser.execute_script("return $('#base-layers-0').get(0).checked"), True)
        self.assertEqual(browser.execute_script("return $('#base-layers-1').get(0).checked"), False)
        # switch base layer
        browser.execute_script("$('#base-layers-1').trigger('click')")
        self.assertEqual(browser.execute_script("return Boolean(%s.baseLayers.Map._map)" % view), False)
        self.assertEqual(browser.execute_script("return Boolean(%s.baseLayers.Satellite._map)" % view), True)
        self.assertEqual(browser.execute_script("return $('#base-layers-0').get(0).checked"), False)
        self.assertEqual(browser.execute_script("return $('#base-layers-1').get(0).checked"), True)
        # ensure choice is mantained
        self._hashchange('#/')
        self._hashchange('#/map')
        self.assertEqual(browser.execute_script("return Boolean(%s.baseLayers.Map._map)" % view), False)
        self.assertEqual(browser.execute_script("return Boolean(%s.baseLayers.Satellite._map)" % view), True)
        self.assertEqual(browser.execute_script("return $('#base-layers-0').get(0).checked"), False)
        self.assertEqual(browser.execute_script("return $('#base-layers-1').get(0).checked"), True)
        # switch back to original base layer
        browser.execute_script("$('#base-layers-0').trigger('click')")
        self.assertEqual(browser.execute_script("return Boolean(%s.baseLayers.Map._map)" % view), True)
        self.assertEqual(browser.execute_script("return Boolean(%s.baseLayers.Satellite._map)" % view), False)
        self.assertEqual(browser.execute_script("return $('#base-layers-0').get(0).checked"), True)
        self.assertEqual(browser.execute_script("return $('#base-layers-1').get(0).checked"), False)
        # ensure choice is mantained
        self._hashchange('#/')
        self._hashchange('#/map')
        self.assertEqual(browser.execute_script("return Boolean(%s.baseLayers.Map._map)" % view), True)
        self.assertEqual(browser.execute_script("return Boolean(%s.baseLayers.Satellite._map)" % view), False)
        self.assertEqual(browser.execute_script("return $('#base-layers-0').get(0).checked"), True)
        self.assertEqual(browser.execute_script("return $('#base-layers-1').get(0).checked"), False)

        # layers control
        browser.execute_script("%s.setView([41.87403473552959, 12.495403289794922], 10)" % self.LEAFLET_MAP)
        panel = browser.find_element_by_css_selector('#fn-map-layers')
        self.assertFalse(panel.is_displayed())
        browser.find_element_by_css_selector('#map-toolbar .icon-layer-2:not(.active)')
        browser.find_element_by_css_selector('#map-toolbar .icon-layer-2').click()
        browser.find_element_by_css_selector('#map-toolbar .icon-layer-2.active')
        self.assertEqual(len(browser.find_elements_by_css_selector('#fn-map-layers .switch-on')), 8)
        self.assertEqual(len(browser.find_elements_by_css_selector('#fn-map-layers .switch-off')), 0)
        self.assertTrue(panel.is_displayed())
        # ensure it doesn't close after clicking on it
        panel.click()
        self.assertTrue(panel.is_displayed())
        # click somewhere else to close
        self.browser.find_element_by_css_selector('#fn-map-layers-mask').click()
        self.assertFalse(panel.is_displayed())
        # ensure it reopens correctly
        browser.find_element_by_css_selector('#map-toolbar .icon-layer-2:not(.active)')
        browser.find_element_by_css_selector('#map-toolbar .icon-layer-2').click()
        sleep(0.1)
        browser.find_element_by_css_selector('#map-toolbar .icon-layer-2.active')
        self.assertTrue(panel.is_displayed())
        # expect the following leaflet layers (unfortunately numbers vary depending on screen size and system...)
        self.assertTrue(2 >= browser.execute_script("return $('#map-js path.marker-potential').length") >= 1)
        self.assertTrue(2 >= browser.execute_script("return $('#map-js path.marker-active').length") >= 1)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-potential').length"), 0)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-active').length"), 1)
        # expect legend counts
        self.assertTrue(5 >= int(browser.find_element_by_css_selector('#legend-item-active .stats').text) >= 4)
        self.assertTrue(3 >= int(browser.find_element_by_css_selector('#legend-item-potential .stats').text) >= 2)

        # fix needed for headled testing
        browser.execute_script('localStorage.clear()')
        browser.refresh()

        # turn off 1 layer
        browser.execute_script('$("#map-control-layer-rome").trigger("click")')
        sleep(0.1)
        self.assertEqual(len(browser.find_elements_by_css_selector('#fn-map-layers .switch-off')), 1)
        self.assertEqual(len(browser.find_elements_by_css_selector('#fn-map-layers .switch-on')), 7)
        # ensure elements have been hidden
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-potential').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-active').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-potential').length"), 0)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-active').length"), 0)
        # expect legend counts
        self.assertEqual(browser.find_element_by_css_selector('#legend-item-active .stats').text, '2')
        self.assertEqual(browser.find_element_by_css_selector('#legend-item-potential .stats').text, '2')
        # turn on again
        browser.execute_script('$("#map-control-layer-rome").trigger("click")')
        # ensure elements have been shown again
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-potential').length"), 2)
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-active').length"), 2)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-potential').length"), 0)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-active').length"), 1)
        # expect legend counts
        self.assertEqual(browser.find_element_by_css_selector('#legend-item-active .stats').text, '5')
        self.assertEqual(browser.find_element_by_css_selector('#legend-item-potential .stats').text, '3')
        # hide again
        browser.execute_script('$("#map-control-layer-rome").trigger("click")')
        # ensure choice is mantained
        self._hashchange('#/')
        self._hashchange('#/map')
        self.assertEqual(len(browser.find_elements_by_css_selector('#fn-map-layers .switch-off')), 1)
        self.assertEqual(len(browser.find_elements_by_css_selector('#fn-map-layers .switch-on')), 7)
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-potential').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-active').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-potential').length"), 0)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-active').length"), 0)
        # test interaction with legend items - issue #171
        browser.execute_script("%s.setView([41.87403473552959, 12.495403289794922], 7)" % self.LEAFLET_MAP)
        browser.find_element_by_css_selector('#legend-item-potential a').click()
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-potential').length"), 0)
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-active').length"), 2)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-potential').length"), 0)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-active').length"), 0)
        self.assertEqual(browser.execute_script("return $('#map-control-legend-potential').bootstrapSwitch('state')"), False)
        # re-enable rome layer
        browser.execute_script('$("#map-control-layer-rome").trigger("click")')
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-potential').length"), 0)
        self.assertTrue(2 >= browser.execute_script("return $('#map-js path.marker-active').length") >= 1)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-potential').length"), 0)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-active').length"), 1)
        browser.find_element_by_css_selector('#legend-item-potential a').click()
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-potential').length"), 1)
        self.assertTrue(2 >= browser.execute_script("return $('#map-js path.marker-active').length") >= 1)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-potential').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-js .cluster.marker-active').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-control-legend-potential').bootstrapSwitch('state')"), True)

        # test map tools panel
        panel = browser.find_element_by_css_selector('#fn-map-tools')
        button = browser.find_element_by_css_selector('#map-toolbar .icon-tools')
        self.assertFalse(panel.is_displayed())
        button.click()
        self.assertTrue(panel.is_displayed())
        # close panel
        self.browser.find_element_by_css_selector('#fn-map-tools-mask').click()
        self.assertFalse(panel.is_displayed())

        # test toolbar hide/show on mobile
        toolbar = browser.find_element_by_css_selector('#map-toolbar')
        button = browser.find_element_by_css_selector('#toggle-toolbar')
        panel = browser.find_element_by_css_selector('#fn-map-layers')
        # open panel
        browser.find_element_by_css_selector('#map-toolbar .icon-layer-2').click()
        self.assertTrue(toolbar.is_displayed())
        self.assertFalse(button.is_displayed())
        self.assertTrue(panel.is_displayed())
        # make window narrower
        window_size = browser.get_window_size()
        browser.set_window_size(400, window_size['height'])
        # ensure toolbar hidden and button shown
        self.assertFalse(toolbar.is_displayed())
        self.assertTrue(button.is_displayed())
        self.assertFalse(panel.is_displayed())
        # show toolbar
        button.click()
        self.assertTrue(toolbar.is_displayed())
        self.assertTrue(button.is_displayed())
        # hide again
        button.click()
        self.assertFalse(toolbar.is_displayed())
        self.assertTrue(button.is_displayed())
        # reset window size to original size
        browser.set_window_size(window_size['width'], window_size['height'])
        # ensure toolbar is visible
        self.assertTrue(toolbar.is_displayed())

    def test_map_legend(self):
        self._reset()
        self._hashchange('#/map')
        browser = self.browser
        browser.execute_script("%s.setView([41.87403473552959, 12.495403289794922], 12)" % self.LEAFLET_MAP)

        # ensure legend is open
        button = browser.find_element_by_css_selector('#btn-legend.disabled')
        legend = browser.find_element_by_css_selector('#map-legend')
        self.assertTrue(legend.is_displayed())

        # ensure counts are correct
        ui_potential = int(browser.find_element_by_css_selector('#legend-item-potential .stats').text)
        ui_planned = int(browser.find_element_by_css_selector('#legend-item-planned .stats').text)
        ui_active = int(browser.find_element_by_css_selector('#legend-item-active .stats').text)
        queryset = Node.objects.published().access_level_up_to('public')
        db_potential = queryset.filter(status__slug='potential').count()
        db_planned = queryset.filter(status__slug='planned').count()
        db_active = queryset.filter(status__slug='active').count()
        self.assertEqual(ui_potential, db_potential)
        self.assertEqual(ui_planned, db_planned)
        self.assertEqual(ui_active, db_active)

        # ensure it can be closed
        browser.find_element_by_css_selector('#map-legend .icon-close').click()
        sleep(0.4)
        self.assertFalse(legend.is_displayed())
        self.assertNotIn('disabled', button.get_attribute('class'))

        # reopen again
        button.click()
        sleep(0.4)
        self.assertIn('disabled', button.get_attribute('class'))
        self.assertTrue(legend.is_displayed())

        # ensure preference is mantained when switching pages back and forth
        button.click()
        sleep(0.4)
        self.assertFalse(legend.is_displayed())
        self._hashchange('#/')
        self._hashchange('#/map')
        legend = browser.find_element_by_css_selector('#map-legend')
        self.assertFalse(legend.is_displayed())

        # reopen legend
        browser.find_element_by_css_selector('#btn-legend').click()
        # ensure groups can be hidden
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-potential').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-active').length"), 2)
        browser.find_element_by_css_selector('#legend-item-active a').click()
        self.assertIn('disabled', browser.find_element_by_css_selector('#legend-item-active').get_attribute('class'))
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-potential').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-active').length"), 0)
        # ensure preference is mantained
        self._hashchange('#/')
        self._hashchange('#/map')
        self.assertIn('disabled', browser.find_element_by_css_selector('#legend-item-active').get_attribute('class'))
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-potential').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-active').length"), 0)
        # ensure it can be re-enabled
        browser.find_element_by_css_selector('#legend-item-active a').click()
        self.assertNotIn('disabled', browser.find_element_by_css_selector('#legend-item-active').get_attribute('class'))
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-potential').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-js path.marker-active').length"), 2)

        # test clustering
        browser.execute_script("%s.setZoom(7)" % self.LEAFLET_MAP)
        self.assertEqual(browser.execute_script("return $('#map-js .marker-active.cluster').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-js .marker-potential.cluster').length"), 1)
        browser.find_element_by_css_selector('#legend-item-active a').click()
        self.assertEqual(browser.execute_script("return $('#map-js .marker-active.cluster').length"), 0)
        self.assertEqual(browser.execute_script("return $('#map-js .marker-potential.cluster').length"), 1)
        browser.find_element_by_css_selector('#legend-item-active a').click()
        self.assertEqual(browser.execute_script("return $('#map-js .marker-active.cluster').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-js .marker-potential.cluster').length"), 1)
        # ensure preference is mantained
        browser.find_element_by_css_selector('#legend-item-active a').click()
        self._hashchange('#/')
        self._hashchange('#/map')
        self.assertEqual(browser.execute_script("return $('#map-js .marker-active.cluster').length"), 0)
        self.assertEqual(browser.execute_script("return $('#map-js .marker-potential.cluster').length"), 1)
        # ensure it can be re-enabled
        browser.find_element_by_css_selector('#legend-item-active a').click()
        self.assertEqual(browser.execute_script("return $('#map-js .marker-active.cluster').length"), 1)
        self.assertEqual(browser.execute_script("return $('#map-js .marker-potential.cluster').length"), 1)

    def test_map_add_node(self):
        browser = self.browser
        leaflet_map = self.LEAFLET_MAP
        self._hashchange('#map')
        # test for a toggleLeafletLayers() bug
        browser.find_element_by_css_selector('#map-toolbar .icon-pin-add').click()
        self._wait_until_element_visible('#signin-modal', 1, 'signin modal not visible')
        self._hashchange('#/')
        self._wait_until_element_visible('#body article a.btn-primary', 2, 'home not visible')
        self._hashchange('#map')
        self._wait_until_element_visible('#map-js', 5, 'map not visible')
        self.assertNotEqual(browser.execute_script("return $('#map-js path').attr('fill-opacity')"), '0.3')
        browser.find_element_by_css_selector('#signin-modal .icon-close').click()
        sleep(0.5)
        # ensure "hidden" elements are visible
        legend = browser.find_element_by_css_selector('#legend-js')
        toolbar = browser.find_element_by_css_selector('#map-toolbar')
        cluster = browser.find_element_by_css_selector('#map-js .cluster')
        self.assertTrue(legend.is_displayed())
        self.assertTrue(toolbar.is_displayed())
        self.assertTrue(cluster.is_displayed())
        # click on add node icon
        browser.find_element_by_css_selector('#map-toolbar .icon-pin-add').click()
        # ensure login box is shown
        self._wait_until_element_visible('#signin-modal', 1, 'signin modal not visible')

        # login as admin
        self._login(open_modal=False)
        self._wait_until_ajax_complete(5, 'timeout while reloading data after login')

        # ensure elements have been hidden
        self.assertFalse(legend.is_displayed())
        self.assertFalse(toolbar.is_displayed())
        cluster = browser.find_element_by_css_selector('#map-js .cluster')
        self.assertFalse(cluster.is_displayed())
        # ensure URL has changed
        self.assertEqual(browser.current_url.split('#')[1], 'map/add')

        # cancel step1
        self.browser.find_element_by_css_selector('#add-node-step1 button').click()
        # ensure hidden element reset to initial state
        self.assertTrue(legend.is_displayed())
        self.assertTrue(toolbar.is_displayed())
        cluster = browser.find_element_by_css_selector('#map-js .cluster')
        self.assertTrue(cluster.is_displayed())

        self.assertEqual(browser.current_url.split('#')[1], 'map')

        # go to step2
        browser.back()
        browser.execute_script("%s.fire('click', { latlng: L.latLng(41.89727804010839, 12.504158020019531) })" % leaflet_map)
        self._wait_until_ajax_complete(5, 'timeout')

        # cancel step2
        self.browser.find_element_by_css_selector('#add-node-step2 .btn-default').click()
        # ensure hidden element reset to initial state
        self.assertTrue(legend.is_displayed())
        self.assertTrue(toolbar.is_displayed())

        # go again to step2
        browser.find_element_by_css_selector('#map-toolbar .icon-pin-add').click()
        browser.execute_script("%s.fire('click', { latlng: L.latLng(41.89727804010839, 12.504158020019531) })" % leaflet_map)
        self._wait_until_ajax_complete(5, 'timeout')

        # confirm
        browser.find_element_by_css_selector('#add-node-step2 .btn-success').click()
        self._wait_until_element_visible('#map-add-node-js', 1, 'add node form not shown in time')
        # add node form
        browser.find_element_by_css_selector('#add-node-form-container .btn-success').click()
        self._wait_until_ajax_complete(5, 'timeout')
        # click on cancel
        browser.find_element_by_css_selector('#add-node-form-container .btn-default').click()
        sleep(0.5)  # animation
        self.assertEqual(len(browser.find_elements_by_css_selector('#add-node-form-container')), 0)

        # ensure hidden element reset to initial state
        self.assertTrue(legend.is_displayed())
        self.assertTrue(toolbar.is_displayed())

        # go again to step2
        browser.find_element_by_css_selector('#map-toolbar .icon-pin-add').click()
        browser.execute_script("%s.fire('click', { latlng: L.latLng(41.89727804010839, 12.504158020019531) })" % leaflet_map)
        self._wait_until_ajax_complete(5, 'timeout')
        # ensure some elements are hidden
        self.assertFalse(legend.is_displayed())
        self.assertFalse(toolbar.is_displayed())
        # cancel step2
        self.browser.find_element_by_css_selector('#add-node-step2 .btn-default').click()
        # ensure hidden elements reset to initial state
        self.assertTrue(legend.is_displayed())
        self.assertTrue(toolbar.is_displayed())

        # log out
        self._logout()

    def test_map_add_node_direct_fragment(self):
        # similar to the previous test but instead of clicking on the add node button
        # we directly open the URL fragment
        self._reset()
        browser = self.browser
        self._hashchange('#map/add')
        self._wait_until_element_visible('#signin-modal', 1, 'signin modal not visible')
        browser.find_element_by_css_selector('#signin-modal .icon-close').click()
        sleep(0.5)
        self._hashchange('#')

        # login as admin
        self._login()
        self._wait_until_ajax_complete(5, 'timeout while reloading data after login')
        self._hashchange('#map/add')

        # ensure elements have been hidden
        legend = browser.find_element_by_css_selector('#legend-js')
        toolbar = browser.find_element_by_css_selector('#map-toolbar')
        cluster = browser.find_element_by_css_selector('#map-js .cluster')
        self.assertFalse(legend.is_displayed())
        self.assertFalse(toolbar.is_displayed())
        self.assertFalse(cluster.is_displayed())
        # ensure URL has changed
        self.assertEqual(browser.current_url.split('#')[1], 'map/add')

        # cancel step1
        self.browser.find_element_by_css_selector('#add-node-step1 button').click()
        # ensure hidden element reset to initial state
        self.assertTrue(legend.is_displayed())
        self.assertTrue(toolbar.is_displayed())
        cluster = browser.find_element_by_css_selector('#map-js .cluster')
        self.assertTrue(cluster.is_displayed())

        self.assertEqual(browser.current_url.split('#')[1], 'map')

        # log out
        self._hashchange('#')
        self._logout()

    def test_map_lat_lng(self):
        LEAFLET_MAP = self.LEAFLET_MAP
        browser = self.browser
        self._hashchange('#map/latlng/41.8625675,12.4931263')
        self.assertEqual(browser.execute_script("return %s.getCenter().lat.toString().substr(0, 8)" % LEAFLET_MAP), "41.86256")
        self.assertEqual(browser.execute_script("return %s.getCenter().lng.toString().substr(0, 8)" % LEAFLET_MAP), "12.49312")
        self.assertEqual(browser.execute_script("return %s.getZoom()" % LEAFLET_MAP), 18)
        self.assertEqual(len(browser.find_elements_by_css_selector('.leaflet-marker-icon.leaflet-zoom-animated.leaflet-clickable')), 1)

    def test_node_list(self):
        self.browser.find_element_by_css_selector('a[href="#nodes"]').click()
        self._wait_until_ajax_complete(5, 'timeout')
        self.assertTrue(self.browser.execute_script("return Ns.body.currentView.$el.attr('id') == 'node-list'"))

        for node in Node.objects.access_level_up_to('public'):
            self.assertIn(node.name, self.browser.page_source)

    def test_node_details(self):
        # ensure not cached
        self.assertTrue(self.browser.execute_script('return Ns.db.nodes.get("pomezia") === undefined'))
        self._hashchange('#nodes/pomezia')
        self.assertTrue(self.browser.execute_script("return Ns.body.currentView.$el.attr('id') == 'map-container'"))
        self.browser.find_element_by_css_selector('#node-details')
        self.assertIn('Pomezia', self.browser.page_source)
        # ensure cached
        self.assertFalse(self.browser.execute_script('return Ns.db.nodes.get("pomezia") === undefined'))

    def test_edit_node(self):
        # login as admin
        self._login()
        # go to edit
        self._hashchange('#nodes/pomezia/edit')
        self._wait_until_element_visible('#node-data-js form', 0.5, 'edit node form not shown in time')
        self.assertNotIn('vienna', self.browser.execute_script('return $("#node-data-js form select[name=layer]").html()'))
        # ensure leaflet popup is open
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#map-js .leaflet-popup')), 1)
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#map-js .leaflet-marker-icon')), 1)

        # go back to node details
        self.browser.find_element_by_css_selector('#node-data-js form .btn-default').click()
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#node-data-js form')), 0)
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#map-js .leaflet-popup')), 0)
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#map-js .leaflet-marker-icon')), 0)
        self.assertIn('nodes/pomezia', self.browser.current_url)

        # go back to edit mode
        self.browser.back()
        self._wait_until_element_visible('#node-data-js form', 0.5, 'edit node form not shown in time')
        # ensure leaflet popup is open
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#map-js .leaflet-popup')), 1)
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#map-js .leaflet-marker-icon')), 1)

        # forward to node details, temporary marker must disappear
        self.browser.forward()
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#map-js .leaflet-popup')), 0)
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#map-js .leaflet-marker-icon')), 0)

        # go back to edit mode and click on save
        self.browser.back()
        self._wait_until_element_visible('#node-data-js form', 0.5, 'edit node form not shown in time')
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#map-js .leaflet-popup')), 1)
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#map-js .leaflet-marker-icon')), 1)
        self.browser.find_element_by_css_selector('#node-data-js .btn-success').click()
        self._wait_until_element_visible('#node-data-js table', 0.5, 'edit node form not shown in time')
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#map-js .leaflet-popup')), 0)
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#map-js .leaflet-marker-icon')), 0)

        # log out
        self._logout()

    def test_user_profile(self):
        self._hashchange('#/users/romano')
        self.assertTrue(self.browser.execute_script("return Ns.body.currentView.$el.attr('id') == 'user-details-container'"))
        self.assertIn('romano', self.browser.page_source)

    def test_general_search(self):
        self._hashchange('#')
        search = self.browser.find_element_by_css_selector('#general-search-input')
        search.send_keys('RD')
        search.send_keys(Keys.ENTER)
        self._wait_until_ajax_complete(5, 'timeout')
        results = self.browser.find_elements_by_css_selector('#js-search-results li')
        self.assertEqual(len(results), 4)
        search = self.browser.find_element_by_css_selector('#general-search-input')
        search.clear()
        search.send_keys('RDP')
        search.send_keys(Keys.ENTER)
        self._wait_until_ajax_complete(5, 'timeout')
        results = self.browser.find_elements_by_css_selector('#js-search-results li')
        self.assertEqual(len(results), 1)
        self.browser.find_element_by_css_selector('#js-search-results li a').click()
        self._wait_until_ajax_complete(5, 'timeout')
        self.assertIn('RDP', self.browser.find_element_by_css_selector('#node-details h2').text)

    def test_general_search_address(self):
        self._hashchange('#')
        search = self.browser.find_element_by_css_selector('#general-search-input')
        search.clear()
        # no results
        search.send_keys('streetnode')
        search.send_keys(Keys.ENTER)
        self._wait_until_ajax_complete(5, 'timeout')
        results = self.browser.find_elements_by_css_selector('#js-search-results li')
        self.assertEqual(len(results), 1)
        self.assertIn('nothing', results[0].text)
        search.send_keys(Keys.ESCAPE)
        search.clear()
        # 1 result
        search.send_keys('via Roma, Pomezia')
        search.send_keys(Keys.ENTER)
        self._wait_until_element_visible('#js-search-results li a.icon-globe', 10, 'no search address result')
        results = self.browser.find_elements_by_css_selector('#js-search-results li')
        self.assertEqual(len(results), 1)
        self.assertNotIn('nothing', results[0].text)
        search.send_keys(Keys.ESCAPE)
        search.clear()
        # 1 result
        search.send_keys('google street')
        search.send_keys(Keys.ENTER)
        self._wait_until_ajax_complete(5, 'timeout')
        results = self.browser.find_elements_by_css_selector('#js-search-results li')
        self.assertEqual(len(results), 1)
        self.assertNotIn('nothing', results[0].text)
        search.send_keys(Keys.ESCAPE)
        search.clear()
        # 2 results
        search.send_keys('VIA delle zoccolette')
        search.send_keys(Keys.ENTER)
        self._wait_until_ajax_complete(5, 'timeout')
        results = self.browser.find_elements_by_css_selector('#js-search-results li')
        self.assertEqual(len(results), 2)
        search.clear()
        # go to first result
        self.browser.find_element_by_css_selector('#js-search-results li a').click()
        self._wait_until_ajax_complete(5, 'timeout')
        self.assertIn('#map/latlng/41.89', self.browser.current_url)

    def test_notifications(self):
        self._reset()
        # login as admin
        self._login()
        self._wait_until_element_visible('#main-actions a.notifications', 1, 'notifications link not visible')
        # open notifications
        self.browser.find_element_by_css_selector('#main-actions a.notifications').click()
        self._wait_until_element_visible('#js-notifications-container .empty', 1, 'notifications panel not visible')
        self.browser.find_element_by_css_selector('#main-actions a.notifications').click()
        # log out
        self._logout()

    def test_password_reset(self):
        self._hashchange('#account/password/reset')
        self._wait_until_element_visible('#password-reset-container', 1, 'password reset container not visible')
        self._login()
        self.assertEqual(self.browser.title, 'Home - Nodeshot')

    def test_layers(self):
        self._hashchange('#layers')
        self._wait_until_element_visible('#layer-list table', 1, 'layers list not visible')
        self.assertEqual(self.browser.title, 'Layer list - Nodeshot')

    def test_layer_details(self):
        self._hashchange('#layers/rome')
        self._wait_until_element_visible('#layer-details table', 1, 'layers details not visible')
        self.assertEqual(self.browser.title, 'Rome - Nodeshot')
        self.assertTrue(self.browser.execute_script('return Ns.body.currentView.map !== undefined'))

    def test_user_nodes(self):
        self._hashchange('#users/romano/nodes')
        self._wait_until_element_visible('#node-list table', 1, 'node list not visible')
        self.assertEqual(self.browser.title, 'Nodes of user: romano - Nodeshot')
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#js-add-node')), 0)

    def test_signup_error(self):
        self._hashchange('#')
        browser = self.browser
        browser.find_element_by_css_selector('a[data-target="#signup-modal"]').click()
        sleep(0.5)
        browser.find_element_by_css_selector('#js-signup-username').send_keys('testing')
        browser.find_element_by_css_selector('#js-signup-email').send_keys('admin@admin.org')
        browser.find_element_by_css_selector('#js-signup-password').send_keys('testing')
        pwd2 = browser.find_element_by_css_selector('#js-signup-password_confirmation')
        pwd2.send_keys('testing')
        pwd2.send_keys(Keys.ENTER)
        self._wait_until_ajax_complete(10, 'Timeout')
        sleep(1)
        # ensure loading indicator is not showing anymore
        self.assertFalse(browser.find_element_by_css_selector('#loading').is_displayed())
        # ensure one error
        self.assertEqual(len(browser.find_elements_by_css_selector('.input-group.hastip.error')), 1)
        self.assertIn('address already exists', browser.find_element_by_css_selector('.tooltip-inner').text)
        browser.find_element_by_css_selector('#signup-modal .icon-close').click()

    def test_my_account(self):
        # not authenticated goes back to home
        self._hashchange('#account')
        self._wait_until_element_visible('#body article.center-stage .btn-primary', 1, 'home page not visible')
        # login
        self._login()
        self._hashchange('#account')
        self._wait_until_element_visible('#account-container', 1, 'account settings not shown after save')
        self.assertEqual(self.browser.find_element_by_css_selector('#account-container h1').text, 'My account')
        self._logout()

    def test_edit_profile(self):
        # not authenticated goes back to home
        self._hashchange('#account/profile/edit')
        self._wait_until_element_visible('#body article.center-stage .btn-primary', 1, 'home page not visible')
        # login
        self._login()
        self._hashchange('#account/profile/edit')
        self._wait_until_element_visible('#form-container', 1, 'form not shown')
        self.browser.find_element_by_css_selector('#form-container .btn-success').click()
        self._wait_until_ajax_complete(5, 'Timeout')
        self._wait_until_element_visible('#account-container', 1, 'account settings not shown after save')
        self.assertEqual(self.browser.find_element_by_css_selector('#account-container h1').text, 'My account')
        self._logout()

    def test_change_password(self):
        # not authenticated goes back to home
        self._hashchange('#account/password/change')
        self._wait_until_element_visible('#body article.center-stage .btn-primary', 1, 'home page not visible')
        # login
        self._login()
        self._hashchange('#account/password/change')
        self._wait_until_element_visible('#form-container', 1, 'form not shown')
        self.assertEqual(self.browser.find_element_by_css_selector('#body article h1').text, 'Change account password')
        self.browser.find_element_by_css_selector('#form-container .btn-success').click()
        self._wait_until_ajax_complete(5, 'Timeout')
        self.assertEqual(len(self.browser.find_elements_by_css_selector('#form-container .has-error')), 2)
        self._logout()



from nodeshot.core.base.utils import check_dependencies

check_dependencies(
    dependencies='nodeshot.networking.net',
    module='nodeshot.networking.hardware'
)


from base import ImageMixin
from manufacturer import Manufacturer
from mac_prefix import MacPrefix
from device_model import DeviceModel
from antenna_model import AntennaModel
from radiation_pattern import RadiationPattern
from antenna import Antenna
from device_to_model_rel import DeviceToModelRel


__all__ = [
    'ImageMixin',
    'Manufacturer',
    'MacPrefix',
    'DeviceModel',
    'AntennaModel',
    'RadiationPattern',
    'Antenna',
    'DeviceToModelRel'
]



from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver
from django.conf import settings

from nodeshot.core.nodes.signals import node_status_changed
from nodeshot.core.nodes.models import Node

from ..tasks import send_message


# ------ NODE CREATED ------ #

@receiver(post_save, sender=Node)
def node_created_handler(sender, **kwargs):
    if kwargs['created']:
        obj = kwargs['instance']
        message = 'node "%s" has been added' % obj.name
        send_message.delay(message)

# ------ NODE STATUS CHANGED ------ #

@receiver(node_status_changed)
def node_status_changed_handler(**kwargs):
    obj = kwargs['instance']
    obj.old_status = kwargs['old_status'].name
    obj.new_status = kwargs['new_status'].name
    message = 'node "%s" changed its status from "%s" to "%s"' % (obj.name, obj.old_status, obj.new_status)
    send_message.delay(message)


# ------ NODE DELETED ------ #

@receiver(pre_delete, sender=Node)
def node_deleted_handler(sender, **kwargs):
    obj = kwargs['instance']
    message = 'node "%s" has been deleted' % obj.name
    send_message.delay(message)


# ------ DISCONNECT UTILITY ------ #

def disconnect():
    """ disconnect signals """
    post_save.disconnect(node_created_handler, sender=Node)
    node_status_changed.disconnect(node_status_changed_handler)
    pre_delete.disconnect(node_deleted_handler, sender=Node)


def reconnect():
    """ reconnect signals """
    post_save.connect(node_created_handler, sender=Node)
    node_status_changed.connect(node_status_changed_handler)
    pre_delete.connect(node_deleted_handler, sender=Node)


from nodeshot.core.base.settings import DISCONNECTABLE_SIGNALS
DISCONNECTABLE_SIGNALS.append(
    {
        'disconnect': disconnect,
        'reconnect': reconnect
    }
)
setattr(settings, 'NODESHOT_DISCONNECTABLE_SIGNALS', DISCONNECTABLE_SIGNALS)



import json
from django.shortcuts import get_object_or_404

from rest_framework.decorators import api_view
from rest_framework.response import Response

from influxdb.client import InfluxDBClientError

from .models import Metric


@api_view(('GET', 'POST'))
def metric_details(request, pk, format=None):
    """
    Get or write metric values
    """
    metric = get_object_or_404(Metric, pk=pk)
    # get
    if request.method == 'GET':
        try:
            results = metric.select(q=request.QUERY_PARAMS.get('q', metric.query))
        except InfluxDBClientError as e:
            return Response(json.loads(e.content), status=e.code)
        return Response(results)
    # post
    else:
        if not request.DATA:
            return Response({'detail': 'expected values in POST data or JSON payload'},
                            status=400)
        data = request.DATA.copy()
        # try converting strings to floats when sending form-data
        if request.content_type != 'application/json':
            for key, value in data.items():
                try:
                    data[key] = float(value) if '.' in value else int(value)
                except ValueError:
                    pass
        # write
        metric.write(data)
        return Response({'detail': 'ok'})



from django.db.models.signals import post_save, pre_delete
from django.dispatch import receiver
from django.contrib.auth import get_user_model
User = get_user_model()

from nodeshot.core.nodes.signals import node_status_changed
from nodeshot.core.nodes.models import Node

from ..settings import settings
from ..models import Notification
from ..tasks import create_notifications


base_queryset = User.objects.filter(is_active=True)


def exclude_owner_of_node(node):
    if node.user_id is not None:
        return base_queryset.exclude(pk=node.user_id)
    else:
        return base_queryset


# ------ NODE CREATED ------ #

@receiver(post_save, sender=Node)
def node_created_handler(sender, **kwargs):
    """ send notification when a new node is created according to users's settings """
    if kwargs['created']:
        obj = kwargs['instance']
        queryset = exclude_owner_of_node(obj)
        create_notifications.delay(**{
            "users": queryset,
            "notification_model": Notification,
            "notification_type": "node_created",
            "related_object": obj
        })


# ------ NODE STATUS CHANGED ------ #

@receiver(node_status_changed)
def node_status_changed_handler(**kwargs):
    """ send notification when the status of a node changes according to users's settings """
    obj = kwargs['instance']
    obj.old_status = kwargs['old_status'].name
    obj.new_status = kwargs['new_status'].name
    queryset = exclude_owner_of_node(obj)
    create_notifications.delay(**{
        "users": queryset,
        "notification_model": Notification,
        "notification_type": "node_status_changed",
        "related_object": obj
    })

    # if node has owner send a different notification to him
    if obj.user is not None:
        create_notifications.delay(**{
            "users": [obj.user],
            "notification_model": Notification,
            "notification_type": "node_own_status_changed",
            "related_object": obj
        })


# ------ NODE DELETED ------ #

@receiver(pre_delete, sender=Node)
def node_deleted_handler(sender, **kwargs):
    """ send notification when a node is deleted according to users's settings """
    obj = kwargs['instance']
    queryset = exclude_owner_of_node(obj)
    create_notifications.delay(**{
        "users": queryset,
        "notification_model": Notification,
        "notification_type": "node_deleted",
        "related_object": obj
    })


# ------ DISCONNECT UTILITY ------ #

def disconnect():
    """ disconnect signals """
    post_save.disconnect(node_created_handler, sender=Node)
    node_status_changed.disconnect(node_status_changed_handler)
    pre_delete.disconnect(node_deleted_handler, sender=Node)


def reconnect():
    """ reconnect signals """
    post_save.connect(node_created_handler, sender=Node)
    node_status_changed.connect(node_status_changed_handler)
    pre_delete.connect(node_deleted_handler, sender=Node)


from nodeshot.core.base.settings import DISCONNECTABLE_SIGNALS
DISCONNECTABLE_SIGNALS.append(
    {
        'disconnect': disconnect,
        'reconnect': reconnect
    }
)
setattr(settings, 'NODESHOT_DISCONNECTABLE_SIGNALS', DISCONNECTABLE_SIGNALS)



import operator

from django.http import Http404
from django.utils.translation import ugettext_lazy as _
from django.utils.text import slugify
from django.db.models import Q
from django.shortcuts import get_object_or_404
from django.contrib.gis.geos import Point, fromstr
from django.core.exceptions import ImproperlyConfigured
from django.conf import settings

from rest_framework import generics, permissions, authentication, status
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.decorators import api_view

from nodeshot.core.nodes.models import Node, Status, Image
from nodeshot.core.layers.models import Layer

from .base import SERVICES, DISCOVERY, MODELS, iso8601_REGEXP
from .settings import STATUS
from .serializers import *


@api_view(['GET'])
def service_discovery(request):
    """
    Open 311 services' discovery.
    """
    return Response(DISCOVERY)


class ServiceDefinitionList(generics.ListAPIView):
    """
    Retrieve list of Open 311 services.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    serializer_class = ServiceListSerializer

    def get(self, request, *args, **kwargs):
        """ return list of open 311 services  """
        # init django rest framework specific stuff
        serializer_class = self.get_serializer_class()
        context = self.get_serializer_context()

        # init empty list
        services = []

        # loop over each service
        for service_type in SERVICES.keys():
            # initialize serializers for layer
            services.append(
                serializer_class(
                    object(),
                    context=context,
                    service_type=service_type
                ).data
            )

        return Response(services)

service_definition_list = ServiceDefinitionList.as_view()


class ServiceDefinitionDetail(APIView):
    """
    Retrieve details of specified service.
    """

    def get(self, request, *args, **kwargs):
        service_type = kwargs['service_type']

        if service_type not in SERVICES.keys():
            return Response({ 'detail': _('Not found') }, status=404)

        serializers = {
            'node': ServiceNodeSerializer,
            'vote': ServiceVoteSerializer,
            'comment': ServiceCommentSerializer,
            'rate': ServiceRatingSerializer,
        }

        # init right serializer
        data = serializers[service_type](object()).data

        return Response(data)

service_definition_detail = ServiceDefinitionDetail.as_view()


def create_output_data(data):
    # convert received geometry to Point
    geom = fromstr(data['geometry']).centroid
    del data['geometry']
    data['long'] = geom[0]
    data['lat'] = geom[1]

    # get status from model and converts it into the mapped status type (open/closed)
    status_id = data['status']
    status = get_object_or_404(Status, pk=status_id)
    data['detailed_status'] = status.name
    data['detailed_status_description'] = status.description
    try:
        data['status'] = STATUS[status.slug]
    except KeyError:
        if settings.DEBUG:
            raise ImproperlyConfigured("NODESHOT['OPEN311']['STATUS'] settings bad configuration: key %s not found"
                                       % status.slug)
        else:
            data['status'] = 'closed'

    return data


class ServiceRequestList(generics.ListCreateAPIView):
    """
    ### GET
    ###Retrieve list of service requests.

    Required parameters:

     * `service_code=<string>`: possible values are: 'node', 'vote', 'rate' , 'comment'

    ####Note:    List is displayed only for 'node' service requests

    Optional  parameters ( relevant only if service_code = node ):

     * `status=<string>`: possible values are: 'open' or 'closed'
     * `start_date=<date>`: date in w3 format, eg 2010-01-01T00:00:00Z
     * `end_date=<date>`: date in w3 format, eg 2010-01-01T00:00:00Z
     * `layer=<layer>`: layer to which node belongs

    ### POST
    ### Create a new service request. Requires authentication.
    See service definition for required parameters
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)
    serializer_class= NodeRequestListSerializer
    model = Node

    def get_serializer(self, instance=None, data=None,
                       files=None, many=False, partial=False):
        """
        Return the serializer instance that should be used for validating and
        deserializing input, and for serializing output.
        """
        serializers = {
            'node': NodeRequestListSerializer,
            'vote': VoteRequestListSerializer,
            'comment': CommentRequestListSerializer,
            'rate': RatingRequestListSerializer,
        }
        context = self.get_serializer_context()
        service_code = context['request'].QUERY_PARAMS.get('service_code', 'node')

        if service_code not in serializers.keys():
            serializer_class = self.get_serializer_class()
        else:
            serializer_class = serializers[service_code]

        return serializer_class(instance, data=data, files=files,
                                many=many, partial=partial, context=context)

    def get_custom_data(self):
        """ additional request.DATA """

        return {
            'user': self.request.user.id
        }

    def get(self, request, *args, **kwargs):
        """ Retrieve list of service requests """
        if 'service_code' not in request.GET.keys():
            return Response({ 'detail': _('A service code must be inserted') }, status=404)

        service_code = request.GET['service_code']

        if service_code not in SERVICES.keys():
            return Response({ 'detail': _('Service not found') }, status=404)

        start_date = None
        end_date = None
        status = None
        layer = None

        STATUSES = {}
        for status_type in ('open', 'closed'):
            STATUSES[status_type] = [k for k, v in STATUS.items() if v == status_type]

        if 'start_date' in request.GET.keys():
            start_date = request.GET['start_date']
            if iso8601_REGEXP.match(start_date) is None:
                return Response({ 'detail': _('Invalid date inserted') }, status=404)

        if 'end_date' in request.GET.keys():
            end_date = request.GET['end_date']
            if iso8601_REGEXP.match(end_date) is None:
                return Response({ 'detail': _('Invalid date inserted') }, status=404)

        if 'status' in request.GET.keys():
            if request.GET['status'] not in ('open','closed'):
                return Response({ 'detail': _('Invalid status inserted') }, status=404)
            status = request.GET['status']

        if 'layer' in request.GET.keys():
            layer = request.GET['layer']
            node_layer = get_object_or_404(Layer, slug=layer)

        service_model = MODELS[service_code]
        if service_code in ('vote', 'comment', 'rate'):
            self.queryset = service_model.objects.none()
        else:
            self.queryset = service_model.objects.all()

            # Filter by layer
            if layer is not None:
                self.queryset = self.queryset.filter(layer = node_layer)

            # Check of date parameters
            if start_date is not None and end_date is not None:
                self.queryset = self.queryset.filter(added__gte = start_date).filter(added__lte = end_date)

            if start_date is not None and end_date is None:
                self.queryset = self.queryset.filter(added__gte = start_date)

            if start_date is None and end_date is not None:
                self.queryset = self.queryset.filter(added__lte = end_date)

        # Check of status parameter
        if status is not None:
            q_list = [Q(status__slug__exact = s) for s in STATUSES[status]]
            self.queryset = self.queryset.filter(reduce(operator.or_, q_list))

        # init right serializer
        kwargs['service_code'] = service_code

        return self.list(request, *args, **kwargs)

    def list(self, request, *args, **kwargs):
        self.object_list = self.filter_queryset(self.get_queryset())
        service_code = kwargs['service_code']

        # Default is to allow empty querysets.  This can be altered by setting
        # `.allow_empty = False`, to raise 404 errors on empty querysets.
        if not self.allow_empty and not self.object_list:
            class_name = self.__class__.__name__
            error_msg = self.empty_error % {'class_name': class_name}
            raise Http404(error_msg)

        # Switch between paginated or standard style responses
        page = self.paginate_queryset(self.object_list)

        if page is not None:
            serializer = self.get_pagination_serializer(page)
        else:
            serializer = self.get_serializer(self.object_list, many=True)

        data = serializer.data

        if service_code == 'node':
            for item in data:
                item = create_output_data(item)
        else:
            pass

        return Response(data)

    def post(self, request, *args, **kwargs):
        """ Post a service request ( requires authentication) """

        service_code = request.POST['service_code']

        if service_code not in SERVICES.keys():
            return Response({ 'detail': _('Service not found') }, status=404)

        serializers = {
            'node': NodeRequestSerializer,
            'vote': VoteRequestSerializer,
            'comment': CommentRequestSerializer,
            'rate': RatingRequestSerializer,
        }

        # init right serializer
        kwargs['service_code'] = service_code
        kwargs['serializer'] = serializers[service_code]

        user=self.get_custom_data()
        request.UPDATED = request.POST.copy()
        request.UPDATED['user'] = user['user']
        if service_code == 'node':
            for checkPOSTdata in ('layer','name','lat','long'):
                # Check if mandatory parameters key exists
                if checkPOSTdata not in request.POST.keys():
                    return Response({ 'detail': _('Mandatory parameter not found') }, status=400)
                else:
                    # Check if mandatory parameters values have been inserted
                    if not request.POST[checkPOSTdata] :
                        return Response({ 'detail': _('Mandatory parameter not found') }, status=400)
            # Get layer id
            layer = Layer.objects.get(slug=request.UPDATED['layer'])
            request.UPDATED['layer'] = layer.id

            # Transform coords in wkt geometry
            lat = float(request.UPDATED['lat'])
            long = float(request.UPDATED['long'])
            point = Point((long, lat))
            request.UPDATED['geometry'] = point.wkt
            request.UPDATED['slug'] = slugify(request.UPDATED['name'])

        return self.create(request, *args, **kwargs)

    def create(self, request, *args, **kwargs):

        serializer = kwargs['serializer']( data=request.UPDATED, files=request.FILES)
        service_code= kwargs['service_code']
        if serializer.is_valid():
            self.pre_save(serializer.object)
            self.object = serializer.save(force_insert=True)
            self.post_save(self.object, created=True,
                           files=request.FILES,
                           service_code=service_code
                           )
            headers = self.get_success_headers(serializer.data)
            response_311 = service_code + '-' + str(self.object.id)
            return Response(response_311, status=status.HTTP_201_CREATED,
                            headers=headers)

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def post_save(self,obj, created, files, service_code):
        if service_code == 'node' and files:
            for file,image_file in files.iteritems():
                image=Image(node=obj, file=image_file)
                image.save()

service_request_list = ServiceRequestList.as_view()


class ServiceRequestDetail(generics.RetrieveAPIView):
    """ Retrieve the details of a service request """

    serializer_class= NodeRequestDetailSerializer

    def get(self, request, *args, **kwargs):
        context = self.get_serializer_context()
        service_code = kwargs['service_code']
        request_id = kwargs['request_id']

        if service_code not in SERVICES.keys():
            return Response({ 'detail': _('Service not found') }, status=404)

        serializers = {
            'node': NodeRequestDetailSerializer,
            'vote': VoteRequestSerializer,
            'comment': CommentRequestSerializer,
            'rate': RatingRequestSerializer,
        }

        service_model = MODELS[service_code]
        request_object = get_object_or_404(service_model, pk=request_id)
        data = serializers[service_code](request_object, context=context).data

        if service_code == 'node':
            data = create_output_data(data)
        else:
            data['status'] = 'Closed'

        data['service_code'] = service_code
        data['service_name'] = SERVICES[service_code]['name']

        return Response(data)

service_request_detail = ServiceRequestDetail.as_view()



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.models import BaseDate
from nodeshot.networking.net.models import Interface


class Vap(BaseDate):
    """
    VAP interface
    represents a virtual wireless interface
    """
    interface = models.ForeignKey('net.Wireless',
                                  verbose_name='wireless interface')
    essid = models.CharField(max_length=50, null=True, blank=True)
    bssid = models.CharField(max_length=50, null=True, blank=True)
    encryption = models.CharField(max_length=50, null=True, blank=True)
    key = models.CharField(max_length=100, null=True, blank=True)
    auth_server = models.CharField(max_length=50, null=True, blank=True)
    auth_port = models.IntegerField(null=True, blank=True)
    accounting_server = models.CharField(max_length=50, null=True, blank=True)
    accounting_port = models.IntegerField(null=True, blank=True)
    
    def __unicode__(self):
        return self.essid
    
    class Meta:
        app_label = 'net'
        db_table = 'net_interface_vap'
        verbose_name = _('vap interface')
        verbose_name_plural = _('vap interfaces')


from rest_framework import serializers

from .models import Inward


__all__ = [
    'InwardMessageSerializer',
]


class InwardMessageSerializer(serializers.ModelSerializer):
    """ status icons """
    
    class Meta:
        model = Inward
        read_only_fields = (
            'status', 'added', 'updated', 'content_type', 'object_id',
            'user', 'ip', 'user_agent', 'accept_language')


from django.contrib.gis.db import models
from django.contrib.gis.geos.collections import GeometryCollection
from django.contrib.gis.geos import GEOSException
from django.utils.translation import ugettext_lazy as _
from django.template.defaultfilters import slugify

from nodeshot.core.base.models import BaseAccessLevel
from nodeshot.core.base.managers import HStoreGeoAccessLevelPublishedManager as NodeManager

from django_hstore.fields import DictionaryField

from ..settings import settings, PUBLISHED_DEFAULT, HSTORE_SCHEMA
from ..signals import node_status_changed
from .status import Status


class Node(BaseAccessLevel):
    """
    Nodes are generic geo-referenced records
    Can be assigned to 'Layers' if nodeshot.core.layers is installed
    Can belong to 'Users'
    """
    name = models.CharField(_('name'), max_length=75, unique=True)
    slug = models.SlugField(max_length=75, db_index=True, unique=True, blank=True)
    status = models.ForeignKey(Status, blank=True, null=True)
    is_published = models.BooleanField(default=PUBLISHED_DEFAULT)

    # TODO: find a way to move this in layers
    if 'nodeshot.core.layers' in settings.INSTALLED_APPS:
        # layer might need to be able to be blank, would require custom validation
        layer = models.ForeignKey('layers.Layer')

    # owner, allow NULL
    user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)

    # geographic information
    geometry = models.GeometryField(
        _('geometry'),
        help_text=_('geometry of the node (point, polygon, line)')
    )
    elev = models.FloatField(_('elevation'), blank=True, null=True)
    address = models.CharField(_('address'), max_length=150, blank=True, null=True)

    # descriptive information
    description = models.TextField(_('description'), max_length=255, blank=True, null=True)
    notes = models.TextField(_('notes'), blank=True, null=True,
                             help_text=_('for internal use only'))

    data = DictionaryField(_('extra data'), null=True, blank=True,
                           editable=False, schema=HSTORE_SCHEMA)

    # manager
    objects = NodeManager()

    # this is needed to check if the status is changing
    # explained here:
    # http://stackoverflow.com/questions/1355150/django-when-saving-how-can-you-check-if-a-field-has-changed
    _current_status = None

    # needed for extensible validation
    _additional_validation = []

    class Meta:
        db_table = 'nodes_node'
        app_label = 'nodes'

    def __unicode__(self):
        return '%s' % self.name

    def __init__(self, *args, **kwargs):
        """ Fill __current_status """
        super(Node, self).__init__(*args, **kwargs)
        # set current status, but only if it is an existing node
        if self.pk:
            self._current_status = self.status_id

    def _autofill_slug(self):
        slugified_name = slugify(self.name)
        # auto generate slug
        if not self.slug or self.slug != slugified_name:
            self.slug = slugified_name

    def clean(self, *args, **kwargs):
        """ call extensible validation """
        self._autofill_slug()
        self.extensible_validation()

    def save(self, *args, **kwargs):
        """
        Custom save method does the following things:
            * converts geometry collections of just 1 item to that item (eg: a collection of 1 Point becomes a Point)
            * intercepts changes to status and fires node_status_changed signal
            * set default status
        """
        # geometry collection check
        if isinstance(self.geometry, GeometryCollection) and 0 < len(self.geometry) < 2:
            self.geometry = self.geometry[0]
        # if no status specified
        if not self.status and not self.status_id:
            try:
                self.status = Status.objects.filter(is_default=True)[0]
            except IndexError:
                pass
        super(Node, self).save(*args, **kwargs)
        # if status of a node changes
        if (self.status and self._current_status and self.status.id != self._current_status) or\
           (self.status_id and self._current_status and self.status_id != self._current_status):
            # send django signal
            node_status_changed.send(
                sender=self.__class__,
                instance=self,
                old_status=Status.objects.get(pk=self._current_status),
                new_status=self.status
            )
        # update _current_status
        self._current_status = self.status_id

    def extensible_validation(self):
        """
        Execute additional validation that might be defined elsewhere in the code.
        Additional validation is introduced through the class method Node.add_validation_method()
        """
        # loop over additional validation method list
        for validation_method in self._additional_validation:
            # call each additional validation method
            getattr(self, validation_method)()

    @classmethod
    def add_validation_method(class_, method):
        """
        Extend validation of Node by adding a function to the _additional_validation list.
        The additional validation function will be called by the clean method

        :method function: function to be added to _additional_validation
        """
        method_name = method.func_name

        # add method name to additional validation method list
        class_._additional_validation.append(method_name)

        # add method to this class
        setattr(class_, method_name, method)

    @property
    def owner(self):
        return self.user

    @property
    def point(self):
        """ returns location of node. If node geometry is not a point a center  point will be returned """
        if not self.geometry:
            raise ValueError('geometry attribute must be set before trying to get point property')
        if self.geometry.geom_type == 'Point':
            return self.geometry
        else:
            try:
                # point_on_surface guarantees that the point is within the geometry
                return self.geometry.point_on_surface
            except GEOSException:
                # fall back on centroid which may not be within the geometry
                # for example, a horseshoe shaped polygon
                return self.geometry.centroid

    if 'grappelli' in settings.INSTALLED_APPS:
        @staticmethod
        def autocomplete_search_fields():
            return ('name__icontains', 'slug__icontains', 'address__icontains')

    # some more properties are added by the layer app
    #  * intersecting_layers



from nodeshot.core.base.utils import choicify


POLARIZATIONS = {
    'horizonal': 1,
    'vertical': 2,
    'circular': 3,
    'linear': 4,
    'dual_linear': 5
}

POLARIZATION_CHOICES = choicify(POLARIZATIONS)



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.models import BaseDate
from nodeshot.networking.net.models import Device, Interface

from choices import POLARIZATION_CHOICES
from . import AntennaModel


class Antenna(BaseDate):
    """ Antenna of a device. A device may have more than one antenna """
    # device foreign key is redundant but it allows us to manage it easily in the django admin
    device = models.ForeignKey(Device)
    model = models.ForeignKey(AntennaModel)
    radio = models.ForeignKey(Interface, blank=True, null=True) #TODO: this should not be blank nor null
    polarization = models.SmallIntegerField(_('Polarization'), choices=POLARIZATION_CHOICES, blank=True, null=True)
    azimuth = models.FloatField(_('azimuth'), blank=True, null=True)
    inclination = models.FloatField(_('inclination'), blank=True, null=True)
    
    # TODO: this must become a postgis point
    # elevation = models.FloatField(_('elevation'), blank=True, null=True)
    # lat = models.FloatField(_('latitude'), blank=True, null=True, help_text=_('automatically inherits the value of the node, specify a different value if needed'))
    # lng = models.FloatField(_('longitude'), blank=True, null=True, help_text=_('automatically inherits the value of the node, specify a different value if needed'))
    
    class Meta:
        app_label = 'hardware'
    
    def __unicode__(self):
        return self.model.__unicode__()
    
    def save(self, *args, **kwargs):
        """
        1. set polarization according to AntennaModel (self.model.polarization) when creating a new antenna
        2. inherit latitude and longitude from node
        """
        if not self.pk and self.model.polarization:
            self.polarization = self.model.polarization
        super(Antenna, self).save(*args, **kwargs)



from rest_framework import pagination, serializers

from .models import *


__all__ = [
    'ServiceListSerializer',
    'ServiceDetailSerializer',
    'PaginatedServiceSerializer',
    'ServiceLoginSerializer',
    'CategorySerializer'
]


class ServiceLoginSerializer(serializers.ModelSerializer):
    class Meta:
        model = ServiceLogin


class ServiceListSerializer(serializers.ModelSerializer):
    """ Service List Serializer  """
    
    details = serializers.HyperlinkedIdentityField(view_name='api_service_details')
    
    class Meta:
        model = Service


class ServiceDetailSerializer(ServiceListSerializer):
    """ Service Detail Serializer  """
    
    logins = ServiceLoginSerializer(many=True, source='servicelogin_set')
    
    class Meta:
        model = Service


class PaginatedServiceSerializer(pagination.PaginationSerializer):
    class Meta:
        object_serializer_class = ServiceListSerializer


class CategorySerializer(serializers.ModelSerializer):
    
    details = serializers.HyperlinkedIdentityField(view_name='api_service_category_details')
    
    class Meta:
        model = Category
        fields = ['id', 'name', 'description', 'added', 'updated', 'details']



from django.conf import settings
from leaflet import app_settings as leaflet_settings


TILESERVER_URL = leaflet_settings['TILES'][0][1]
MAP_ZOOM = leaflet_settings['DEFAULT_ZOOM']
MAP_CENTER = list(leaflet_settings['DEFAULT_CENTER'])
LEAFLET_OPTIONS = {
    'fillOpacity': 0.7,
    'opacity': 1,
    'dashArray': None,
    'lineCap': None,
    'lineJoin': None,
    'radius': 6,
    # when adding new nodes other leaflet layers are dimmed to the following opacity:
    'temporaryOpacity': 0.3
}
LEAFLET_OPTIONS = getattr(settings, 'NODESHOT_UI_LEAFLET_OPTIONS', LEAFLET_OPTIONS)
DISABLE_CLUSTERING_AT_ZOOM = getattr(settings, 'NODESHOT_UI_DISABLE_CLUSTERING_AT_ZOOM', 12)
MAX_CLUSTER_RADIUS = getattr(settings, 'NODESHOT_UI_MAX_CLUSTER_RADIUS', 90)
DATETIME_FORMAT = getattr(settings, 'NODESHOT_UI_DATETIME_FORMAT', 'dd MMMM yyyy, HH:mm')
DATE_FORMAT = getattr(settings, 'NODESHOT_UI_DATE_FORMAT', 'dd MMMM yyyy')
ADDRESS_SEARCH_TRIGGERS = [
    ',',
    'st.',
    ' street',
    ' square',
    ' road',
    ' avenue',
    ' lane',
    'footpath',
    'via ',
    'viale ',
    'piazza ',
    'strada ',
    'borgo ',
    'contrada ',
    'zona ',
    'fondo ',
    'vico ',
    'sentiero ',
    'plaza ',
    ' plaza',
    'calle ',
    'avenida '
]
ADDRESS_SEARCH_TRIGGERS = getattr(settings, 'NODESHOT_UI_ADDRESS_SEARCH_TRIGGERS', ADDRESS_SEARCH_TRIGGERS)
LOGO = getattr(settings, 'NODESHOT_UI_LOGO', None)

if 'nodeshot.community.participation' in settings.INSTALLED_APPS:
    VOTING_ENABLED = getattr(settings, 'NODESHOT_UI_VOTING_ENABLED', True)
    RATING_ENABLED = getattr(settings, 'NODESHOT_UI_RATING_ENABLED', True)
    COMMENTS_ENABLED = getattr(settings, 'NODESHOT_UI_COMMENTS_ENABLED', True)
else:
    VOTING_ENABLED = False
    RATING_ENABLED = False
    COMMENTS_ENABLED = False

if 'nodeshot.community.mailing' in settings.INSTALLED_APPS:
    CONTACTING_ENABLED = getattr(settings, 'NODESHOT_UI_CONTACTING_ENABLED', True)
else:
    CONTACTING_ENABLED = False

if 'social_auth' in settings.INSTALLED_APPS:
    FACEBOOK_ENABLED = bool(settings.FACEBOOK_APP_ID) and bool(settings.FACEBOOK_API_SECRET)  # noqa
    GOOGLE_ENABLED = bool(settings.GOOGLE_OAUTH2_CLIENT_ID) and bool(settings.GOOGLE_OAUTH2_CLIENT_SECRET)  # noqa
    GITHUB_ENABLED = bool(settings.GITHUB_APP_ID) and bool(settings.GITHUB_API_SECRET)  # noqa
    SOCIAL_AUTH_ENABLED = FACEBOOK_ENABLED or GOOGLE_ENABLED or GITHUB_ENABLED  # noqa
else:
    FACEBOOK_ENABLED = False
    GOOGLE_ENABLED = False
    GITHUB_ENABLED = False
    SOCIAL_AUTH_ENABLED = False

if 'nodeshot.core.websockets' in settings.INSTALLED_APPS:
    from nodeshot.core.websockets import DOMAIN, PATH, PORT

    WEBSOCKETS = {
        'DOMAIN': DOMAIN,
        'PATH': PATH,
        'PORT': PORT
    }
else:
    WEBSOCKETS = False

from nodeshot.community.profiles.settings import REGISTRATION_OPEN  # noqa

GOOGLE_ANALYTICS_UA = getattr(settings, 'NODESHOT_UI_GOOGLE_ANALYTICS_UA', None)
GOOGLE_ANALYTICS_OPTIONS = getattr(settings, 'NODESHOT_UI_GOOGLE_ANALYTICS_OPTIONS', 'auto')

PIWIK_ANALYTICS_BASE_URL = getattr(settings, 'NODESHOT_UI_PIWIK_ANALYTICS_BASE_URL', None)
PIWIK_ANALYTICS_SITE_ID = getattr(settings, 'NODESHOT_UI_PIWIK_ANALYTICS_SITE_ID', None)

# additional geojson sources that are not contained in the main /api/v1/nodes.geojson response
ADDITIONAL_GEOJSON_URLS = getattr(settings, 'NODESHOT_UI_ADDITIONAL_GEOJSON_URLS', [])

MAP_3D_ENABLED = getattr(settings, 'NODESHOT_UI_MAP_3D_ENABLED', True)
MAP_TOOLS_ENABLED = getattr(settings, 'NODESHOT_UI_MAP_TOOLS_ENABLED', True)
MAP_PREFERENCES_ENABLED = getattr(settings, 'NODESHOT_UI_MAP_PREFERENCES_ENABLED', True)

PRIVACY_POLICY_LINK = getattr(settings, 'NODESHOT_UI_PRIVACY_POLICY_LINK', '#/pages/privacy-policy')
TERMS_OF_SERVICE_LINK = getattr(settings, 'NODESHOT_UI_TERMS_OF_SERVICE_LINK', '#/pages/terms-of-service')

METRICS_ENABLED = 'nodeshot.core.metrics' in settings.INSTALLED_APPS



from django.conf import settings
from django.utils.translation import ugettext_lazy as _


OUTWARD_MAXLENGTH = getattr(settings, 'NODESHOT_MAILING_OUTWARD_MAXLENGTH', 9999)
OUTWARD_MINLENGTH = getattr(settings, 'NODESHOT_MAILING_OUTWARD_MINLENGTH', 50)
OUTWARD_SCHEDULING = getattr(settings, 'NODESHOT_MAILING_OUTWARD_SCHEDULING', (
    ('00', _('midnight')),
    ('04', _('04:00 AM')),
))
OUTWARD_HTML = getattr(settings, 'NODESHOT_MAILING_OUTWARD_HTML', True)
OUTWARD_STEP = getattr(settings, 'NODESHOT_MAILING_OUTWARD_STEP', 20)
OUTWARD_DELAY = getattr(settings, 'NODESHOT_MAILING_OUTWARD_DELAY', 10)

INWARD_REQUIRE_AUTH = getattr(settings, 'NODESHOT_MAILING_INWARD_REQUIRE_AUTH', True)
INWARD_MAXLENGTH = getattr(settings, 'NODESHOT_MAILING_INWARD_MAXLENGTH', 2000)
INWARD_MINLENGTH = getattr(settings, 'NODESHOT_MAILING_INWARD_MINLENGTH', 15)
INWARD_LOG = getattr(settings, 'NODESHOT_MAILING_INWARD_LOG', True)



from django.conf.urls import patterns, url
from django.conf import settings


urlpatterns = patterns('',)


if settings.DEBUG:
    urlpatterns += patterns('nodeshot.core.websockets.views',
        url(r'^test/$', 'test', name='websocket_test'),
    )



from django.test import TestCase
from django.core import mail, management
from django.contrib.gis.geos import Point

from nodeshot.core.nodes.models import Node
from nodeshot.core.layers.models import Layer
from nodeshot.networking.net.models import Device, Interface, Ip, Vap, Wireless
from nodeshot.networking.links.models import Link
from nodeshot.community.profiles.models import Profile as User
from nodeshot.community.profiles.models import EmailAddress, EmailConfirmation
from nodeshot.community.mailing.models import Inward

from .models import *  # noqa
from . import settings


class TestOldImporter(TestCase):
    fixtures = [
        'initial_data.json',
        'test_profiles.json',
        'test_layers.json',
        'test_status.json',
    ]

    mysql_fixtures = [
        'test_oldusers.json',
        'test_oldnodes.json',
        'test_olddevices.json',
        'test_oldlinks.json',
        'test_oldcontacts.json'
    ]

    def setUp(self):
        for fixture in self.mysql_fixtures:
            management.call_command('loaddata', fixture, database='old_nodeshot')

        # setup layer 1 and 2 area
        l1 = Layer.objects.get(pk=1)
        l1.area = 'POLYGON ((11.9727761230500001 42.0001370328649983, 12.3188454589879992 42.2669596835330026, 12.9148537597690005 42.1509994246999966, 13.2609230957069997 41.8079840923359995, 12.7500588378940005 41.3951648168940025, 12.5962502441440005 41.4445954826359966, 12.4067360839879992 41.6501514845399967, 12.1925026855510001 41.7444877274350006, 12.1128518066440005 41.9184405084070022, 12.0277077636760001 41.9368313579400009, 12.0277077636760001 41.9429604629400004, 11.9727761230500001 42.0001370328649983))'
        l1.save()

        l2 = l1 = Layer.objects.get(pk=2)
        l2.area = 'POLYGON ((10.1833779296829992 43.9387627493410022, 10.3454262695289998 43.9901623726480011, 10.6475502929680008 43.9822576330009980, 10.7491738281280007 43.9071101075910022, 10.8205849609349993 43.7188257293139984, 10.7574135742189991 43.5558293167039992, 10.6283242187470002 43.3484662663620028, 10.4470498046879996 43.3344836017819972, 10.3591591796899998 43.4602116419999973, 10.2712685546909999 43.5518482711390007, 10.2575356445299999 43.6830839058609968, 10.2383095703089992 43.8338488741229995, 10.1751381835920007 43.9328291618219993, 10.1833779296829992 43.9387627493410022))'
        l2.save()

        l = Layer()
        l.id = 5
        l.name = 'Default Layer'
        l.slug = 'default-layer'
        l.description = 'Default Layer'
        l.organization = 'Test'
        l.published = True
        l.area = Point(40.0, 10.0)
        l.full_clean()
        l.save()

    def test_command(self):
        for user in User.objects.all():
            user.delete()

        settings.DEFAULT_LAYER = 5
        management.call_command('import_old_nodeshot', noinput=True)

        nodes = Node.objects.all().order_by('id')
        devices = Device.objects.all().order_by('id')
        interfaces = Interface.objects.all().order_by('id')
        ip_addresses = Ip.objects.all().order_by('id')
        links = Link.objects.all().order_by('id')
        users = User.objects.all()
        email_addresses = EmailAddress.objects.all()
        email_confirmations = EmailConfirmation.objects.all()
        inwards = Inward.objects.all()

        self.assertEqual(len(nodes), 5)
        self.assertEqual(len(devices), 2)
        self.assertEqual(len(interfaces), 4)
        self.assertEqual(len(ip_addresses), 8)
        self.assertEqual(Vap.objects.count(), 2)
        self.assertEqual(len(links), 1)
        self.assertEqual(len(users), 6)
        self.assertEqual(len(email_addresses), 6)
        self.assertEqual(len(email_confirmations), 0)
        self.assertEqual(len(mail.outbox), 0)
        self.assertEqual(len(inwards), 1)

        # admin check
        self.assertEqual(User.objects.filter(is_staff=True).count(), 2)
        self.assertEqual(User.objects.filter(is_superuser=True).count(), 1)
        superuser = User.objects.filter(is_superuser=True).first()
        self.assertEqual(superuser.email, 'oldnode1@test.com')
        self.assertEqual(superuser.is_staff, True)
        self.assertEqual(superuser.password, 'md5$C2xwUqHfBz4P$f37f9e6f4f076b1239a5c5dff2c8a2ab')
        # staff user
        randomuser = User.objects.get(username='randomuser')
        self.assertEqual(randomuser.email, 'random@test.com')
        self.assertEqual(randomuser.is_staff, True)
        self.assertEqual(randomuser.is_superuser, False)
        self.assertEqual(randomuser.password, 'md5$C2xwUqHfBz4P$f37f9e6f4f076b1239a5c5dff2c8a2ac')
        # date joined check
        self.assertEqual(randomuser.date_joined.strftime('%Y-%m-%dT%H:%M:%S'), '2013-05-18T13:36:47')

        # node1
        self.assertEqual(nodes[0].id, 1)
        self.assertEqual(nodes[0].name, 'oldnode1 rome')
        self.assertEqual(nodes[0].slug, 'oldnode1-rome')
        self.assertEqual(nodes[0].user.get_full_name(), 'Oldnode1 Owner')
        self.assertEqual(nodes[0].user.username, 'oldnode1-owner')
        self.assertEqual(nodes[0].user.email, 'oldnode1@test.com')
        self.assertEqual(nodes[0].description, 'oldnode1-description')
        self.assertEqual(nodes[0].data['postal_code'], '00185')
        self.assertEqual(nodes[0].status.slug, 'active')
        self.assertEqual(nodes[0].updated.strftime('%Y-%m-%dT%H:%M:%S'), '2013-06-20T13:36:47')
        self.assertEqual(nodes[0].added.strftime('%Y-%m-%dT%H:%M:%S'), '2013-06-14T13:30:29')
        self.assertEqual(nodes[0].geometry[0], 12.5390629470348003)
        self.assertEqual(nodes[0].geometry[1], 41.9064152946931969)
        self.assertEqual(nodes[0].elev, 24.5)
        # ensure layer has been picked correctly
        self.assertEqual(nodes[0].layer.slug, 'rome')
        # ensure owner date_joined is correct
        self.assertEqual(nodes[0].user.date_joined.strftime('%Y-%m-%dT%H:%M:%S'), '2013-06-14T13:30:29')

        # node2
        self.assertEqual(nodes[1].id, 2)
        self.assertEqual(nodes[1].name, 'oldnode2 rome')
        self.assertEqual(nodes[1].slug, 'oldnode2-rome')
        self.assertEqual(nodes[1].user.get_full_name(), 'Oldnode2 Owner')
        self.assertEqual(nodes[1].user.username, 'oldnode2-owner')
        self.assertEqual(nodes[1].user.email, 'oldnode2@test.com')
        self.assertEqual(nodes[1].description, 'oldnode2-description')
        self.assertEqual(nodes[1].data['postal_code'], '00175')
        self.assertEqual(nodes[1].data['is_hotspot'], 'true')
        self.assertEqual(nodes[1].status.slug, 'active')
        self.assertEqual(nodes[1].updated.strftime('%Y-%m-%dT%H:%M:%S'), '2013-06-19T13:36:47')
        self.assertEqual(nodes[1].added.strftime('%Y-%m-%dT%H:%M:%S'), '2013-06-19T13:30:29')
        self.assertEqual(nodes[1].geometry[0], 12.534556835889)
        self.assertEqual(nodes[1].geometry[1], 41.90746129417)
        self.assertEqual(nodes[1].elev, 15.5)
        # ensure layer has been picked correctly
        self.assertEqual(nodes[1].layer.slug, 'rome')
        # ensure owner date_joined is correct
        self.assertEqual(nodes[1].user.date_joined.strftime('%Y-%m-%dT%H:%M:%S'), '2013-06-19T13:30:29')

        # node3
        self.assertEqual(nodes[2].id, 3)
        self.assertEqual(nodes[2].name, 'oldnode3 pisa')
        self.assertEqual(nodes[2].slug, 'oldnode3-pisa')
        self.assertEqual(nodes[2].user.get_full_name(), 'Oldnode3 Pisano')
        self.assertEqual(nodes[2].user.username, 'oldnode3-pisano')
        self.assertEqual(nodes[2].user.email, 'oldnode3@test.com')
        self.assertEqual(nodes[2].description, 'oldnode3 description')
        self.assertEqual(nodes[2].data['postal_code'], '00100')
        self.assertEqual(nodes[2].status.slug, 'potential')
        self.assertEqual(nodes[2].updated.strftime('%Y-%m-%dT%H:%M:%S'), '2013-06-18T13:36:47')
        self.assertEqual(nodes[2].added.strftime('%Y-%m-%dT%H:%M:%S'), '2013-06-18T13:30:29')
        self.assertEqual(nodes[2].geometry[0], 10.3973981737999992)
        self.assertEqual(nodes[2].geometry[1], 43.7175764660999988)
        self.assertEqual(nodes[2].elev, 10)
        # ensure layer has been picked correctly
        self.assertEqual(nodes[2].layer.slug, 'pisa')
        # ensure owner date_joined is correct
        self.assertEqual(nodes[2].user.date_joined.strftime('%Y-%m-%dT%H:%M:%S'), '2013-06-18T13:30:29')

        # ensure default layer
        self.assertEqual(nodes[3].layer.slug, 'default-layer')
        self.assertEqual(nodes[3].layer.id, 5)

        # device import check
        self.assertEqual(devices[0].id, 1)
        self.assertEqual(devices[0].name, 'device1')
        self.assertEqual(devices[0].node_id, 1)
        self.assertEqual(devices[0].description, 'device1-description')
        self.assertEqual(devices[0].type, 'radio')
        self.assertEqual(devices[0].data['cname'], 'oldnode1-device1')
        self.assertEqual(devices[0].data['model'], 'test model')
        self.assertEqual(devices[0].added.strftime('%Y-%m-%dT%H:%M:%S'), '2013-08-14T13:30:29')
        self.assertEqual(devices[0].routing_protocols.count(), 1)
        self.assertEqual(devices[0].routing_protocols.first().name, 'olsr')

        # interface import check
        self.assertEqual(interfaces[0].id, 1)
        self.assertEqual(interfaces[0].device_id, 1)
        self.assertEqual(interfaces[0].mac, '00:27:22:38:d1:38')
        self.assertEqual(interfaces[0].name, 'eth')
        self.assertEqual(interfaces[0].get_type_display(), 'ethernet')
        self.assertEqual(interfaces[0].added.strftime('%Y-%m-%dT%H:%M:%S'), '2013-08-15T13:30:29')
        self.assertEqual(interfaces[0].ip_set.count(), 2)
        ipv4 = interfaces[0].ip_set.filter(protocol='ipv4').first()
        self.assertEqual(str(ipv4.address), '10.40.0.1')
        ipv6 = interfaces[0].ip_set.filter(protocol='ipv6').first()
        self.assertEqual(str(ipv6.address), '2001:4c00:893b:fede:eddb:decd:e878:88b3')

        # wireless interface check
        wireless_interface = Wireless.objects.get(pk=interfaces[1].id)
        self.assertEqual(wireless_interface.id, 2)
        self.assertEqual(wireless_interface.device_id, 1)
        self.assertEqual(wireless_interface.mac, '00:27:22:38:d1:39')
        self.assertEqual(wireless_interface.name, 'wifi')
        self.assertEqual(wireless_interface.get_type_display(), 'wireless')
        self.assertEqual(wireless_interface.channel, '5620')
        self.assertEqual(wireless_interface.mode, 'ap')
        self.assertEqual(wireless_interface.added.strftime('%Y-%m-%dT%H:%M:%S'), '2013-08-16T13:30:29')
        ipv4 = wireless_interface.ip_set.filter(protocol='ipv4').first()
        self.assertEqual(str(ipv4.address), '172.16.40.27')
        ipv6 = wireless_interface.ip_set.filter(protocol='ipv6').first()
        self.assertEqual(str(ipv6.address), '2001:4c00:893b:fede:eddb:decd:e878:88b4')

        # vap check
        vap = wireless_interface.vap_set.first()
        self.assertEqual(vap.essid, 'essid test')
        self.assertEqual(vap.bssid, 'bssidtest')

        # link check
        self.assertEqual(links[0].id, 1)
        self.assertEqual(links[0].interface_a_id, 2)
        self.assertEqual(links[0].interface_b_id, 4)
        self.assertEqual(links[0].metric_type, 'etx')
        self.assertEqual(links[0].metric_value, 1)
        self.assertEqual(links[0].dbm, -76)
        self.assertEqual(links[0].min_rate, 50)
        self.assertEqual(links[0].max_rate, 50)

        # inward contact check
        self.assertEqual(inwards[0].id, 1)
        self.assertEqual(inwards[0].object_id, 1)
        self.assertEqual(inwards[0].status, 1)
        self.assertEqual(inwards[0].from_name, 'Tester')
        self.assertEqual(inwards[0].from_email, 'tester@test.com')
        self.assertEqual(inwards[0].message, 'This is a test old contact')
        self.assertEqual(inwards[0].ip, '10.40.0.56')
        self.assertEqual(inwards[0].user_agent, 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/33.0.1750.152 Chrome/33.0.1750.152 Safari/537.36')
        self.assertEqual(inwards[0].accept_language, 'it-IT,it;q=0.8,en-US;q=0.6,en;q=0.4')
        self.assertEqual(inwards[0].added.strftime('%Y-%m-%dT%H:%M:%S'), '2013-09-14T13:30:29')

        # --- update --- #

        n = OldNode(**{
            "name": "addednode1",
            "slug": "addednode1",
            "owner": "addednode1 owner",
            "description": "addednode1-description",
            "postal_code": "00185",
            "email": "addednode@test.com",
            "password": "",
            "lat": 41.4064152946931969,
            "lng": 12.7390629470348003,
            "alt": 23.5,
            "status": "a"
        })
        n.save()

        d = OldDevice(**{
            "name": "addeddevice1",
            "node": n,
            "cname": "addeddevice1",
            "description": "addeddevice1-description",
            "type": "test model",
            "routing_protocol": "olsr"
        })
        d.save()

        i = OldInterface(**{
            "device": d,
            "mac_address": "00:27:22:38:D1:48",
            "ipv4_address": "10.40.0.6",
            "ipv6_address": "2001:4c00:893b:fede:eddb:decd:e878:88c3",
            "cname": "addedeth",
            "type": "eth",
            "status": "r"
        })
        i.save()

        l = OldLink(**{
            "from_interface": OldInterface.objects.first(),
            "to_interface": i,
            "etx": 1,
            "dbm": -75,
            "sync_tx": 60,
            "sync_rx": 40
        })
        l.save()

        c = OldContact(**{
            "node_id": 1,
            "from_name": "Added",
            "from_email": "added@test.com",
            "message": "This is an added test old contact",
            "ip": "10.40.0.57",
            "user_agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Ubuntu Chromium/33.0.1750.152 Chrome/33.0.1750.152 Safari/537.36",
            "http_referer": "http://map.ninux.org/",
            "accept_language": "it-IT,it;q=0.8,en-US;q=0.6,en;q=0.4"
        })
        c.save()

        management.call_command('import_old_nodeshot', noinput=True)

        nodes = Node.objects.all().order_by('id')
        devices = Device.objects.all().order_by('id')
        interfaces = Interface.objects.all().order_by('id')
        ip_addresses = Ip.objects.all().order_by('id')
        links = Link.objects.all().order_by('id')
        users = User.objects.all().order_by('id')
        email_addresses = EmailAddress.objects.all()
        email_confirmations = EmailConfirmation.objects.all()
        inwards = Inward.objects.all().order_by('id')

        self.assertEqual(len(nodes), 6)
        self.assertEqual(len(devices), 3)
        self.assertEqual(len(interfaces), 5)
        self.assertEqual(len(ip_addresses), 10)
        self.assertEqual(Vap.objects.count(), 2)
        self.assertEqual(len(links), 2)
        self.assertEqual(len(users), 7)
        self.assertEqual(len(email_addresses), 7)
        self.assertEqual(len(email_confirmations), 0)
        self.assertEqual(len(inwards), 2)

        # admin check
        self.assertEqual(User.objects.filter(is_staff=True).count(), 2)
        self.assertEqual(User.objects.filter(is_superuser=True).count(), 1)

        self.assertEqual(nodes[0].user.username, 'oldnode1-owner')
        self.assertEqual(nodes[1].user.username, 'oldnode2-owner')
        self.assertEqual(nodes[2].user.username, 'oldnode3-pisano')
        self.assertEqual(nodes[3].user.username, 'oldnode4-default')
        self.assertEqual(nodes[4].user.username, 'oldnode1-owner2')
        self.assertEqual(nodes[5].user.username, 'addednode1-owner')
        self.assertEqual(nodes[5].user.email, 'addednode@test.com')
        self.assertEqual(nodes[5].name, 'addednode1')
        self.assertEqual(nodes[5].description, 'addednode1-description')

        device = Device.objects.last()
        self.assertEqual(device.name, 'addeddevice1')

        interface = Interface.objects.last()
        self.assertEqual(interface.name, 'addedeth')
        self.assertEqual(interface.ip_set.count(), 2)

        link = Link.objects.last()
        self.assertEqual(link.interface_b_id, i.id)
        self.assertEqual(link.dbm, -75)

        self.assertEqual(inwards[1].from_name, 'Added')

        # ------ try to cause troubles ------ #

        ot = OldNode(**{
            "name": "troublingnode",
            "slug": "troublingnode",
            "owner": "troublemaker",
            "description": "troublingnode-description",
            "postal_code": "00185",
            "email": "troublemaker@test.com",
            "password": "",
            "lat": 42.4064152946931969,
            "lng": 13.7390629470348003,
            "alt": 23.5,
            "status": "a"
        })
        ot.save()

        troublemaker = User(**{
            "first_name": "Trouble",
            "last_name": "Maker",
            "username": "icausetroubles",
            "email": "troublemaker@test.com"
        })
        troublemaker.save()

        management.call_command('import_old_nodeshot', noinput=True, verbosity=2)

        self.assertEqual(Node.objects.filter(name='troublingnode').count(), 1)



"""
reusable restframework mixins for API views
"""

import reversion
from rest_framework.response import Response


class ACLMixin(object):
    """ implements ACL in views """

    def get_queryset(self):
        """
        Returns only objects which are accessible to the current user.
        If user is not authenticated all public objects will be returned.

        Model must implement AccessLevelManager!
        """
        return self.queryset.accessible_to(user=self.request.user)


class CustomDataMixin(object):
    """
    Implements custom data in views

    Must implement:
        * self.serializer_custom_class: a custom serializer
        * self.get_custom_data(): method that specifies the custom data to pass to the custom serializer
    """

    def get_custom_data(self):
        """ automatically determine user on creation """
        raise NotImplementedError('CustomDataMixin needs a get_custom_data method')

    def get_custom_serializer(self, **kwargs):
        """ returns the custom serializer class """
        try:
            serializer_class = self.serializer_custom_class
        except AttributeError:
            serializer_class = self.get_serializer

        return serializer_class(**kwargs)

    def create(self, request, *args, **kwargs):
        """ custom create method """
        # copy request.DATA
        data = request.DATA.copy()

        # get the additional data
        additional_data = self.get_custom_data()

        # merge the two
        custom_data = dict(data.items() + additional_data.items())

        # pass custom data to serializer_custom_class
        serializer = self.get_custom_serializer(data=custom_data,
                                                files=request.FILES,
                                                context=self.get_serializer_context())

        if serializer.is_valid():
            self.pre_save(serializer.object)
            self.object = serializer.save(force_insert=True)
            self.post_save(self.object, created=True)
            headers = self.get_success_headers(serializer.data)
            return Response(serializer.data, status=201, headers=headers)

        return Response(serializer.errors, status=400)


class RevisionUpdate(object):
    """
    Mixin that adds compatibility with django reversion for PUT and PATCH requests
    """

    def put(self, request, *args, **kwargs):
        """ custom put method to support django-reversion """
        with reversion.create_revision():
            reversion.set_user(request.user)
            reversion.set_comment('changed through the RESTful API from ip %s' % request.META['REMOTE_ADDR'])
            return self.update(request, *args, **kwargs)

    def patch(self, request, *args, **kwargs):
        """ custom patch method to support django-reversion """
        with reversion.create_revision():
            reversion.set_user(request.user)
            reversion.set_comment('changed through the RESTful API from ip %s' % request.META['REMOTE_ADDR'])
            kwargs['partial'] = True
            return self.update(request, *args, **kwargs)


class RevisionCreate(object):
    """
    Mixin that adds compatibility with django reversion for POST requests
    """

    def post(self, request, *args, **kwargs):
        """ custom put method to support django-reversion """
        with reversion.create_revision():
            reversion.set_user(request.user)
            reversion.set_comment('created through the RESTful API from ip %s' % request.META['REMOTE_ADDR'])
            return self.create(request, *args, **kwargs)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models, connection


class Migration(SchemaMigration):

    def forwards(self, orm):
        if 'links_link' not in connection.introspection.table_names():
            # Adding model 'Link'
            db.create_table(u'links_link', (
                (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
                ('added', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime(2015, 2, 28, 0, 0))),
                ('updated', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime(2015, 2, 28, 0, 0))),
                ('access_level', self.gf('django.db.models.fields.SmallIntegerField')(default=0)),
                ('type', self.gf('django.db.models.fields.SmallIntegerField')(default=1, max_length=10, null=True, blank=True)),
                ('interface_a', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='link_interface_from', null=True, to=orm['net.Interface'])),
                ('interface_b', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='link_interface_to', null=True, to=orm['net.Interface'])),
                ('node_a', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='link_node_from', null=True, to=orm['nodes.Node'])),
                ('node_b', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='link_node_to', null=True, to=orm['nodes.Node'])),
                ('line', self.gf('django.contrib.gis.db.models.fields.LineStringField')(null=True, blank=True)),
                ('status', self.gf('django.db.models.fields.SmallIntegerField')(default=0)),
                ('first_seen', self.gf('django.db.models.fields.DateTimeField')(default=None, null=True, blank=True)),
                ('last_seen', self.gf('django.db.models.fields.DateTimeField')(default=None, null=True, blank=True)),
                ('metric_type', self.gf('django.db.models.fields.CharField')(max_length=6, null=True, blank=True)),
                ('metric_value', self.gf('django.db.models.fields.FloatField')(null=True, blank=True)),
                ('max_rate', self.gf('django.db.models.fields.IntegerField')(default=None, null=True, blank=True)),
                ('min_rate', self.gf('django.db.models.fields.IntegerField')(default=None, null=True, blank=True)),
                ('dbm', self.gf('django.db.models.fields.IntegerField')(default=None, null=True, blank=True)),
                ('noise', self.gf('django.db.models.fields.IntegerField')(default=None, null=True, blank=True)),
                ('data', self.gf(u'django_hstore.fields.DictionaryField')(null=True, blank=True)),
                ('shortcuts', self.gf(u'django_hstore.fields.ReferencesField')(null=True, blank=True)),
            ))
            db.send_create_signal('links', ['Link'])


    def backwards(self, orm):
        # Deleting model 'Link'
        db.delete_table(u'links_link')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'layers.layer': {
            'Meta': {'object_name': 'Layer'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'}),
            'area': ('django.contrib.gis.db.models.fields.GeometryField', [], {}),
            'data': (u'django_hstore.fields.DictionaryField', [], {'null': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_external': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'mantainers': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['profiles.Profile']", 'symmetrical': 'False', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '50'}),
            'new_nodes_allowed': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'nodes_minimum_distance': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'organization': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50'}),
            'text': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        'links.link': {
            'Meta': {'object_name': 'Link'},
            'access_level': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'}),
            'data': (u'django_hstore.fields.DictionaryField', [], {'null': 'True', 'blank': 'True'}),
            'dbm': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'first_seen': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'interface_a': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'link_interface_from'", 'null': 'True', 'to': "orm['net.Interface']"}),
            'interface_b': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'link_interface_to'", 'null': 'True', 'to': "orm['net.Interface']"}),
            'last_seen': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'line': ('django.contrib.gis.db.models.fields.LineStringField', [], {'null': 'True', 'blank': 'True'}),
            'max_rate': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'metric_type': ('django.db.models.fields.CharField', [], {'max_length': '6', 'null': 'True', 'blank': 'True'}),
            'metric_value': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'min_rate': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'node_a': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'link_node_from'", 'null': 'True', 'to': "orm['nodes.Node']"}),
            'node_b': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'link_node_to'", 'null': 'True', 'to': "orm['nodes.Node']"}),
            'noise': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'shortcuts': (u'django_hstore.fields.ReferencesField', [], {'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'type': ('django.db.models.fields.SmallIntegerField', [], {'default': '1', 'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'})
        },
        'net.device': {
            'Meta': {'object_name': 'Device'},
            'access_level': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'}),
            'data': (u'django_hstore.fields.DictionaryField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'elev': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'first_seen': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_seen': ('django.db.models.fields.DateTimeField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'location': ('django.contrib.gis.db.models.fields.PointField', [], {'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'node': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['nodes.Node']"}),
            'notes': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'os': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'os_version': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'routing_protocols': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['net.RoutingProtocol']", 'symmetrical': 'False', 'blank': 'True'}),
            'shortcuts': (u'django_hstore.fields.ReferencesField', [], {'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.SmallIntegerField', [], {'default': '2', 'max_length': '2'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'})
        },
        'net.interface': {
            'Meta': {'object_name': 'Interface'},
            'access_level': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'}),
            'data': (u'django_hstore.fields.DictionaryField', [], {'null': 'True', 'blank': 'True'}),
            'device': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['net.Device']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mac': ('netfields.fields.MACAddressField', [], {'default': 'None', 'max_length': '17', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'mtu': ('django.db.models.fields.IntegerField', [], {'default': '1500', 'null': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '10', 'null': 'True', 'blank': 'True'}),
            'rx_rate': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'shortcuts': (u'django_hstore.fields.ReferencesField', [], {'null': 'True', 'blank': 'True'}),
            'tx_rate': ('django.db.models.fields.IntegerField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'type': ('django.db.models.fields.IntegerField', [], {'max_length': '2', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'})
        },
        'net.routingprotocol': {
            'Meta': {'unique_together': "(('name', 'version'),)", 'object_name': 'RoutingProtocol', 'db_table': "'net_routing_protocol'"},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'}),
            'version': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'})
        },
        'nodes.node': {
            'Meta': {'object_name': 'Node'},
            'access_level': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'}),
            'address': ('django.db.models.fields.CharField', [], {'max_length': '150', 'null': 'True', 'blank': 'True'}),
            'data': (u'django_hstore.fields.DictionaryField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'elev': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'geometry': ('django.contrib.gis.db.models.fields.GeometryField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'layer': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['layers.Layer']"}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '75'}),
            'notes': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '75', 'blank': 'True'}),
            'status': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['nodes.Status']", 'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['profiles.Profile']", 'null': 'True', 'blank': 'True'})
        },
        'nodes.status': {
            'Meta': {'ordering': "['order']", 'object_name': 'Status'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'fill_color': ('nodeshot.core.base.fields.RGBColorField', [], {'max_length': '7', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_default': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '75'}),
            'stroke_color': ('nodeshot.core.base.fields.RGBColorField', [], {'default': "'#000000'", 'max_length': '7', 'blank': 'True'}),
            'stroke_width': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'text_color': ('nodeshot.core.base.fields.RGBColorField', [], {'default': "'#FFFFFF'", 'max_length': '7', 'blank': 'True'})
        },
        'profiles.profile': {
            'Meta': {'object_name': 'Profile'},
            'about': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'address': ('django.db.models.fields.CharField', [], {'max_length': '150', 'blank': 'True'}),
            'birth_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'country': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2015, 2, 28, 0, 0)'}),
            'email': ('django.db.models.fields.EmailField', [], {'db_index': 'True', 'unique': 'True', 'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'gender': ('django.db.models.fields.CharField', [], {'max_length': '1', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '254', 'db_index': 'True'})
        }
    }

    complete_apps = ['links']



from .node import Node
from .image import Image
from .status import Status


__all__ = [
    'Node',
    'Image',
    'Status'
]


# ------ Signals ------ #


from django.dispatch import receiver
from django.db.models.signals import pre_delete, post_save
from django.core.cache import cache
from ..signals import node_status_changed


@receiver(post_save, sender=Status)
@receiver(pre_delete, sender=Status)
@receiver(post_save, sender=Node)
@receiver(pre_delete, sender=Node)
@receiver(node_status_changed, sender=Node)
def clear_cache(sender, **kwargs):
    # clear only cached pages if supported
    if hasattr(cache, 'delete_pattern'):
        cache.delete_pattern('views.decorators.cache.cache*')
    # otherwise clear the entire cache
    else:
        cache.clear()



from django.http import Http404
from django.utils.translation import ugettext_lazy as _
from django.db.models import Q

from rest_framework import authentication, generics

from nodeshot.core.base.mixins import ACLMixin
from nodeshot.core.nodes.models import Node

from .serializers import *
from .models import *


class LinkList(ACLMixin, generics.ListAPIView):
    """
    Retrieve link list according to user access level

    Parameters:

     * `limit=<n>`: specify number of items per page (defaults to 40)
     * `limit=0`: turns off pagination
    """
    authentication_classes = (authentication.SessionAuthentication,)
    queryset = Link.objects.all()
    serializer_class = LinkListSerializer
    pagination_serializer_class = PaginatedLinkSerializer
    paginate_by_param = 'limit'
    paginate_by = 40

link_list = LinkList.as_view()


class LinkGeoJSONList(ACLMixin, generics.ListAPIView):
    """
    Retrieve link list in GeoJSON format
    """
    authentication_classes = (authentication.SessionAuthentication,)
    queryset = Link.objects.all()
    serializer_class = LinkListGeoJSONSerializer

link_geojson_list = LinkGeoJSONList.as_view()


class LinkDetails(ACLMixin, generics.RetrieveAPIView):
    """
    Retrieve details of specified link
    """
    authentication_classes = (authentication.SessionAuthentication,)
    queryset = Link.objects.all()
    serializer_class = LinkDetailSerializer

link_details = LinkDetails.as_view()


class LinkGeoJSONDetails(LinkDetails):
    """
    Get link details in GeoJSON format
    """
    authentication_classes = (authentication.SessionAuthentication,)
    queryset = Link.objects.all()
    serializer_class = LinkDetailGeoJSONSerializer

link_geojson_details = LinkDetails.as_view()


class NodeLinkList(generics.ListAPIView):
    """
    Retrieve links of specified node according to user access level.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    serializer_class = LinkDetailSerializer

    def initial(self, request, *args, **kwargs):
        """
        Custom initial method:
            * ensure node exists and store it in an instance attribute
            * change queryset to return only links of current node
        """
        super(NodeLinkList, self).initial(request, *args, **kwargs)
        # ensure node exists
        try:
            self.node = Node.objects.published()\
                                    .accessible_to(request.user)\
                                    .get(slug=self.kwargs.get('slug', None))
        except Node.DoesNotExist:
            raise Http404(_('Node not found.'))
        # check permissions on node (for link creation)
        self.check_object_permissions(request, self.node)
        # return only links of current node
        self.queryset = Link.objects.select_related('node')\
                                    .accessible_to(self.request.user)\
                                    .filter(Q(node_a_id=self.node.id) |
                                            Q(node_b_id=self.node.id))

node_link_list = NodeLinkList.as_view()



from django.conf import settings
from django.contrib.auth import get_user_model
User = get_user_model()

from rest_framework import serializers, pagination

from nodeshot.core.nodes.models import Node
from nodeshot.community.profiles.serializers import ProfileRelationSerializer
from .models import Comment, Vote, Rating, NodeParticipationSettings, NodeRatingCount


__all__ = [
    'CommentAddSerializer',
    'CommentListSerializer',
    'CommentSerializer',
    'NodeCommentSerializer',
    'ParticipationSerializer',
    'NodeParticipationSerializer',
    'RatingListSerializer',
    'RatingAddSerializer',
    'VoteListSerializer',
    'VoteAddSerializer',
    'LinksSerializer',
    'PaginationSerializer',
    'NodeParticipationSettingsSerializer',
    'NodeSettingsSerializer',
    'LayerParticipationSettingsSerializer',
    'LayerSettingsSerializer'
]


class LinksSerializer(serializers.Serializer):
    next = pagination.NextPageField(source='*')
    prev = pagination.PreviousPageField(source='*')


class PaginationSerializer(pagination.BasePaginationSerializer):
    links = LinksSerializer(source='*')  # Takes the page object as the source
    total_results = serializers.Field(source='paginator.count')
    results_field = 'nodes'


class CommentAddSerializer(serializers.ModelSerializer):
    class Meta:
        model = Comment
        fields = ('node', 'user', 'text')


class CommentListSerializer(serializers.ModelSerializer):
    """ Comment serializer """
    node = serializers.Field(source='node.name')
    username = serializers.Field(source='user.username')

    class Meta:
        model = Comment
        fields = ('node', 'username', 'text', 'added')
        read_only_fields = ('added',)


class CommentSerializer(serializers.ModelSerializer):
    username = serializers.Field(source='user.username')

    class Meta:
        model = Comment
        fields = ('username', 'text', 'added',)


class NodeCommentSerializer(serializers.ModelSerializer):
    comments = CommentSerializer(source='comment_set')

    class Meta:
        model = Node
        fields = ('name', 'description', 'comments')


class CommentRelationSerializer(serializers.ModelSerializer):
    """ display user info """
    user = ProfileRelationSerializer(source='user')

    class Meta:
        model = Comment
        fields = ('user', 'text', 'added',)


class RatingAddSerializer(serializers.ModelSerializer):
    class Meta:
        model = Rating
        fields = ('node', 'user', 'value', )


class RatingListSerializer(serializers.ModelSerializer):
    """ Rating serializer """
    node = serializers.Field(source='node.name')
    username = serializers.Field(source='user.username')

    class Meta:
        model = Rating
        fields = ('node', 'username', 'value',)
        read_only_fields = ('added',)


class VoteAddSerializer(serializers.ModelSerializer):
    class Meta:
        model = Vote
        fields = ('node', 'user', 'vote', )


class VoteListSerializer(serializers.ModelSerializer):
    """ Votes serializer """
    node = serializers.Field(source='node.name')
    username = serializers.Field(source='user.username')

    class Meta:
        model = Vote
        fields = ('node', 'username', 'vote',)
        read_only_fields = ('added',)


class ParticipationSerializer(serializers.ModelSerializer):
    class Meta:
        model = NodeRatingCount
        fields = ('likes', 'dislikes', 'rating_count',
                  'rating_avg', 'comment_count')


class NodeParticipationSerializer(serializers.ModelSerializer):
    """ Node participation details """
    participation = ParticipationSerializer(source='noderatingcount')

    class Meta:
        model = Node
        fields = ('name', 'slug', 'address', 'participation')


class NodeSettingsSerializer(serializers.ModelSerializer):
    class Meta:
        model = NodeParticipationSettings
        fields = ('voting_allowed', 'rating_allowed', 'comments_allowed',)


class NodeParticipationSettingsSerializer(serializers.ModelSerializer):
    """ Node participation settings """
    participation_settings = NodeSettingsSerializer(source='node_participation_settings')

    class Meta:
        model = Node
        fields = ('name', 'slug', 'address', 'participation_settings')


if 'nodeshot.core.layers' in settings.INSTALLED_APPS:
    from .models import LayerParticipationSettings

    class LayerSettingsSerializer(serializers.ModelSerializer):
        class Meta:
            model = LayerParticipationSettings
            fields = ('voting_allowed', 'rating_allowed', 'comments_allowed',)

    # noqa
    class LayerParticipationSettingsSerializer(serializers.ModelSerializer):
        """ Layer participation settings"""
        participation_settings = LayerSettingsSerializer(source='layer_participation_settings')

        class Meta:
            model = Node
            fields = ('name', 'slug', 'participation_settings')


# ------ Add relationship to ExtensibleNodeSerializer ------ #

from nodeshot.core.nodes.serializers import ExtensibleNodeSerializer

ExtensibleNodeSerializer.add_relationship(
    'comments',
    serializer=CommentRelationSerializer,
    many=True,
    queryset=lambda obj, request: obj.comment_set.all()
)

ExtensibleNodeSerializer.add_relationship(
    'counts',
    serializer=ParticipationSerializer,
    queryset=lambda obj, request: obj.noderatingcount
)

ExtensibleNodeSerializer.add_relationship(
    'votes_url',
    view_name='api_node_votes',
    lookup_field='slug'
)

ExtensibleNodeSerializer.add_relationship(
    'ratings_url',
    view_name='api_node_ratings',
    lookup_field='slug'
)

ExtensibleNodeSerializer.add_relationship(
    'comments_url',
    view_name='api_node_comments',
    lookup_field='slug'
)


def voted(obj, request):
    """
    Determines if current logged-in user has already voted on a node
    returns 1 if user has already liked
    returns -1 if user has already disliked
    returns False if user hasn't voted or if not authenticated
    """
    if request.user.is_authenticated():
        v = Vote.objects.filter(node_id=obj.id, user_id=request.user.id)
        if len(v) > 0:
            return v[0].vote
    # hasn't voted yet or not authenticated
    return False

ExtensibleNodeSerializer.add_relationship(
    'voted',
    function=voted
)

ExtensibleNodeSerializer.add_relationship(
    'voting_allowed',
    function=lambda obj, request: obj.voting_allowed
)

ExtensibleNodeSerializer.add_relationship(
    'rating_allowed',
    function=lambda obj, request: obj.rating_allowed
)

ExtensibleNodeSerializer.add_relationship(
    'comments_allowed',
    function=lambda obj, request: obj.comments_allowed
)



from django.conf import settings
from django.utils.translation import ugettext_lazy as _


LOGIN_TYPES = (
    (1, _('read-only')),
    (2, _('write')),
)

SERVICE_STATUS = (
    (1, _('up')),
    (2, _('down')),
    (3, _('not reachable'))
)

APPLICATION_PROTOCOLS = (
    ('http', 'http'),
    ('https', 'https'),
    ('ftp', 'FTP'),
    ('smb', 'Samba'),
    ('afp', 'AFP'),
    ('git', 'Git'),
)

TRANSPORT_PROTOCOLS = (
    ('tcp', 'TCP'),
    ('udp', 'UDP'),
)



from django.test import TestCase
from django.core.urlresolvers import reverse
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _

from nodeshot.core.base.tests import user_fixtures
from nodeshot.core.base.choices import ACCESS_LEVELS

from .models import *


class CMSTest(TestCase):
    fixtures = [
        'initial_data.json',
        user_fixtures,
    ]

    def test_page_list(self):
        url = reverse('api_page_list')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.data), Page.objects.count())

    def test_page_detail(self):
        url = reverse('api_page_detail', args=['privacy-policy'])
        response = self.client.get(url)
        self.assertContains(response, 'privacy-policy')

    def test_menu_list(self):
        url = reverse('api_menu_list')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_menu_list_check_nested(self):
        url = reverse('api_menu_list')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        # ensure there are the correct amount of children in the JSON response
        got_children = MenuItem.objects.get(name='About')
        self.assertEqual(len(response.data[got_children.id-1]['children']), got_children.menuitem_set.count())
    
    def test_menu_list_check_acl(self):
        url = reverse('api_menu_list')
        # ensure link is public
        response = self.client.get(url)
        self.assertIn('Software Credits', response.content)
        # set about menu item as restricted access
        item = MenuItem.objects.get(name='Software Credits')
        item.access_level = ACCESS_LEVELS.get('trusted')
        item.save()
        # ensure menu item is public anymore
        response = self.client.get(url)
        self.assertNotIn('Software Credits', response.content)
        # ensure admin can see it
        self.client.login(username='admin', password='tester')
        response = self.client.get(url)
        self.assertIn('Software Credits', response.content)

    def test_nested_menu_item(self):
        new = MenuItem()
        new.name = 'new'
        new.url = '#'
        new.full_clean()  # validation should be ok

        # set a parent which already has a parent
        new.parent = MenuItem.objects.exclude(parent=None).first()
        with self.assertRaises(ValidationError):
            new.full_clean()
